SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_log_error]') AND type in (N'P', N'PC'))
drop procedure mdm_log_error
GO

CREATE PROCEDURE [mdm_log_error]
(		
	@param_table_name VARCHAR(50),
	@param_service_name VARCHAR(50),
	@param_record_id INT,
	@param_record_id_from VARCHAR(30),
	@param_error_source VARCHAR(100),
	@param_error_code VARCHAR(10),
	@param_error_message VARCHAR(4000),
	@param_error_dttm DATETIME
) AS 
BEGIN
    BEGIN TRAN InnerTran
	BEGIN TRY
		 INSERT INTO [mdm_error_log]
		 (
		   TABLE_NAME,
		   SERVICE_NAME,
		   RECORD_ID,
		   RECORD_ID_FROM,
		   ERROR_SOURCE,
		   ERROR_CODE,
		   ERROR_MESSAGE,
		   ERROR_DATE
		   ) 
		 VALUES
		 (
		   @param_table_name,
		   @param_service_name,
		   @param_record_id,
		   @param_record_id_from,
		   @param_error_source,
		   @param_error_code,
		   @param_error_message,
		   @param_error_dttm
		  )
	COMMIT TRAN InnerTran
	END TRY
	BEGIN CATCH
		ROLLBACK TRAN InnerTran
	END CATCH
END
GO
-- END LOG ERROR UTIL

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'mdm_get_sequence_next') AND type in (N'P', N'PC'))
 drop procedure MDM_GET_SEQUENCE_NEXT
GO

-- this procedure returns a value for use by other stored procedures; it
-- does not return a result set, and so it not appropriate for use in 
-- architect jobs. For Architect jobs, use mdm_get_sequence
CREATE PROCEDURE [mdm_get_sequence_next]  
		(
		@parm_table_name VARCHAR(50)
		)
	AS 
	SET NOCOUNT ON
 	DECLARE @seq_curr_val INT 
    DECLARE @strSQL nvarchar(100)
 	
	BEGIN TRY
	
		SET @strSQL=N'insert '+@parm_table_name+'_SEQ default values SELECT @seq_curr_val= SCOPE_IDENTITY()'		
        EXECUTE SP_Executesql @strsql,N'@seq_curr_val INTEGER OUTPUT', @seq_curr_val OUTPUT
        
        RETURN @seq_curr_val
	END TRY
	BEGIN CATCH
		ROLLBACK TRAN

		DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									' : '+@errorSeverityString+
									' : '+@mdm_error_status+
									' : '+@errorStateString
		-- Log Error
		EXEC [mdm_log_error] 'MDM_TABLE_SEQUENCE',null,0,null,'mdm_get_sequence_next',@mdm_error_status,@mdm_error_messages,@errorDate

		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
	END CATCH
GO

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'mdm_get_sequence') AND type in (N'P', N'PC'))
 drop procedure MDM_GET_SEQUENCE
GO

CREATE PROCEDURE [mdm_get_sequence]  
		(
		@parm_table_name VARCHAR(50)
		)
	AS
	BEGIN 
		SET NOCOUNT ON
 		DECLARE @seq_curr_val INT 
		EXEC @seq_curr_val=[mdm_get_sequence_next] @parm_table_name
		select @seq_curr_val as seq_curr_val
	END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_getStats]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_getStats]
GO

CREATE PROCEDURE [mdm_getStats] (@parm_entity_type       VARCHAR(25),
								 @parm_stat_location     VARCHAR(25),
								 @parm_staging_id		 INT = NULL) AS
  DECLARE @table_name VARCHAR(50);
  DECLARE @entitySt_tbl VARCHAR(50);
  DECLARE @stg_tbl     VARCHAR(50);
  DECLARE @audit_table VARCHAR(50);
  DECLARE @strSql     NVARCHAR(200);
BEGIN
  BEGIN TRY
    IF (@parm_staging_id IS NULL)
     BEGIN
       SET @entitySt_tbl = 'MDM_'+@parm_entity_type+'_ST' 
       SET @stg_tbl = 'MDM_STAGING'
     END
    ELSE 
     BEGIN
       SET @entitySt_tbl = 'MDM_BULK_ST_'+CONVERT(VARCHAR,@parm_staging_id)
       SET @stg_tbl = 'MDM_STAGING_' +CONVERT(VARCHAR,@parm_staging_id)
     END
     
    IF (@parm_stat_location = 'POST_CLUSTER')
      BEGIN
        UPDATE STATISTICS MDM_ENTITY_CLUSTERS 
        SET @strSql = 'UPDATE STATISTICS '+@entitySt_tbl
        EXECUTE sp_executesql @strSql       
        SET @strSql = 'UPDATE STATISTICS '+@stg_tbl
        EXECUTE sp_executesql @strSql        
      END       
    
    SET @table_name = 'MDM_'+@parm_entity_type+'_TT'
    SET @audit_table = 'MDM_'+@parm_entity_type+'_AL'
    IF (@parm_stat_location = 'POST_LIVE')
      BEGIN
        UPDATE STATISTICS MDM_ENTITY_CLUSTERS 
        SET @strSql = 'UPDATE STATISTICS '+@audit_table
        EXECUTE sp_executesql @strSql       
        SET @strSql = 'UPDATE STATISTICS '+@table_name
        EXECUTE sp_executesql @strSql        
      END   
  END TRY
  BEGIN CATCH
        DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)
		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									     ' : '+@errorSeverityString+
									     ' : '+@mdm_error_status+
									     ' : '+@errorStateString
		-- Log Error		     
		EXEC [mdm_log_error] @table_name,null,null,null,'mdm_getStats',@mdm_error_status,@mdm_error_messages,@errorDate

		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
  END CATCH
END

GO

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[stwd_entityCluster_r]') AND type in (N'P', N'PC'))
 drop procedure stwd_entityCluster_r
GO

CREATE PROCEDURE [stwd_entityCluster_r]
(
	@parm_cluster_id INT,
	@parm_entity_type VARCHAR(23),
	@parm_userid NVARCHAR(513),
	@retire_dttm DATETIME
)
AS
	DECLARE @tbl_nm VARCHAR(50);
	DECLARE @cluster_count  INT;
	DECLARE @actual_retire_dttm DATETIME;
    DECLARE @alreadyRetired INT;
    DECLARE @strSql NVARCHAR(500);

	SET NOCOUNT ON
	BEGIN TRY
      BEGIN TRAN	
	   SET @tbl_nm = 'MDM_'+@parm_entity_type+'_TT'
	
	   SET @strSql = 'SELECT @cluster_count = count(1) FROM '+@tbl_nm+'
                       WHERE mdm_entity_cluster_id = @clusterId
                         AND (mdm_valid_to_dttm IS NULL OR mdm_valid_to_dttm > @retireDttm)';
       EXECUTE sp_executesql @strSql,N'@clusterId int,@retireDttm datetime,@cluster_count INTEGER OUTPUT',@parm_cluster_id,@retire_dttm,@cluster_count OUTPUT
          
       IF (@cluster_count = 0)
           BEGIN
              SET @strSql = 'SELECT @actual_retire_dttm = MAX(mdm_valid_to_dttm) FROM '+@tbl_nm+'
                              WHERE mdm_entity_cluster_id = @clusterId';
              EXECUTE sp_executesql @strSql,N'@clusterId int,@actual_retire_dttm DATETIME OUTPUT',@parm_cluster_id,@actual_retire_dttm OUTPUT 
           END
       ELSE
           BEGIN           
		        SET @strSql = 'UPDATE '+@tbl_nm+'
                                  SET mdm_valid_to_dttm = @retire_dttm
                                WHERE mdm_entity_cluster_id = @clusterId
                                  AND (mdm_valid_to_dttm IS NULL OR mdm_valid_to_dttm > @retire_dttm)';
		        EXECUTE sp_executesql @strSql,N'@retire_dttm DATETIME,@clusterId int',@retire_dttm,@parm_cluster_id
		
		        SET @tbl_nm='MDM_'+@parm_entity_type+'_AL'
		        
		        SET @strSql = 'INSERT INTO '+@tbl_nm+' (TABLE_NAME,RECORD_ID,USER_ACTION,MDM_MODIFIED_BY,MODIFIED_DTTM)
                                 SELECT ''MDM_'+@parm_entity_type+'_TT'',mdm_entity_id,''RETIRE'',@userId,@retire_dttm
                                   FROM mdm_'+@parm_entity_type+'_tt
                                  WHERE mdm_entity_cluster_id = @clusterId
                                    AND mdm_valid_to_dttm = @retire_dttm';
                EXECUTE sp_executesql @strSql,N'@userId NVARCHAR(513),@retire_dttm DATETIME,@clusterId int',@parm_userid,@retire_dttm,@parm_cluster_id
				        
                SET @tbl_nm = 'MDM_FORCED_CLUSTERS';

                UPDATE mdm_forced_clusters
                   SET original_cluster_id = -1
                 WHERE original_cluster_id = @parm_cluster_id;
     
                UPDATE mdm_forced_clusters
                   SET current_cluster_id = -1
                 WHERE current_cluster_id = @parm_cluster_id;
                      
                SET @tbl_nm = 'MDM_ENTITY_REL';
                     
                UPDATE mdm_entity_rel
                   SET mdm_valid_to_dttm = @retire_dttm
                WHERE (mdm_from_cluster_id = @parm_cluster_id OR mdm_to_cluster_id = @parm_cluster_id);
           END 
     COMMIT TRAN 
	END TRY
	BEGIN CATCH
		ROLLBACK TRAN
		
		DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									' : '+@errorSeverityString+
									' : '+@mdm_error_status+
									' : '+@errorStateString
		-- Log Error
		EXEC [mdm_log_error] @tbl_nm,null,@parm_cluster_id,'mdm_cluster_id','stwd_entityCluster_r',@mdm_error_status,@mdm_error_messages,@errorDate

		
		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
	END CATCH
GO

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[stwd_entity_r]') AND type in (N'P', N'PC'))
 drop procedure stwd_entity_r
GO

CREATE PROCEDURE [stwd_entity_r]
(
	@parm_mdm_entity_id INT,
	@parm_entity_type VARCHAR(23),
	@parm_userid NVARCHAR(513),
	@retire_dttm DATETIME
)
AS
	DECLARE @tbl_nm NVARCHAR(50);
	DECLARE @strSql NVARCHAR(500);
	SET NOCOUNT ON
	BEGIN TRY	
       BEGIN TRAN
          SET @tbl_nm='MDM_'+@parm_entity_type+'_TT'
			
		  SET @strSql = 'UPDATE '+@tbl_nm+'
                            SET mdm_valid_to_dttm = @retire_dttm
                          WHERE mdm_entity_id = @entityId';
          EXECUTE sp_executesql @strSql,N'@retire_dttm DATETIME,@entityId int',@retire_dttm,@parm_mdm_entity_id
		  
		  SET @tbl_nm='MDM_'+@parm_entity_type+'_AL'
		  
		  SET @strSql = 'INSERT INTO '+@tbl_nm+' (TABLE_NAME,RECORD_ID,USER_ACTION,MDM_MODIFIED_BY,MODIFIED_DTTM)
                            VALUES (''MDM_'+@parm_entity_type+'_TT'',@entityId,''RETIRE'',@userId,@retire_dttm)';
          EXECUTE sp_executesql @strSql,N'@entityId int,@userId NVARCHAR(513),@retire_dttm DATETIME',@parm_mdm_entity_id,@parm_userid,@retire_dttm                            		  
      COMMIT TRAN    
	END TRY

	BEGIN CATCH
		ROLLBACK TRAN

		DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									' : '+@errorSeverityString+
									' : '+@mdm_error_status+
									' : '+@errorStateString
		-- Log Error
		--NOTE Do we log old or new id?
		EXEC [mdm_log_error] @tbl_nm,null,@parm_mdm_entity_id,'mdm_entity_id','stwd_entity_r',@mdm_error_status,@mdm_error_messages,@errorDate

		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
	END CATCH

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_entity_r]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_entity_r]
GO

CREATE PROCEDURE [mdm_entity_r]
(
	@parm_mdm_entity_id INT,
	@parm_entity_type VARCHAR(23),
	@parm_userid NVARCHAR(513),
	@retire_dttm DATETIME
)
AS
	DECLARE @strSql NVARCHAR(500);
	DECLARE @active_contributors INT;
	DECLARE @is_survivor CHAR(1);
	DECLARE @cluster_id INT;
	DECLARE @actual_retire_date DATETIME;
    DECLARE @target_dttm DATETIME;
	SET NOCOUNT ON
	BEGIN TRY	
	  BEGIN TRAN
	    IF (@retire_dttm is null)
          SET @target_dttm=getdate();
        ELSE
          SET @target_dttm=@retire_dttm;
        
        -- Check first: if the entity has already been retired, then return retired date
        SET @strSql = 'SELECT @actual_retire_date = mdm_valid_to_dttm FROM mdm_'+@parm_entity_type+'_tt
                        WHERE mdm_entity_id = @entityId';
        EXECUTE sp_executesql @strSql,N'@entityId int,@actual_retire_date DATETIME OUTPUT',@parm_mdm_entity_id,@actual_retire_date OUTPUT

	   IF (@actual_retire_date IS NULL or @actual_retire_date > @retire_dttm) 		   
		   BEGIN
			  SET @strSql = 'SELECT @cluster_id = mdm_entity_cluster_id,
			                        @is_survivor = mdm_survivor
                               FROM mdm_'+@parm_entity_type+'_tt
                              WHERE mdm_entity_id = @entityId';
              EXECUTE sp_executesql @strSql,N'@entityId int,@cluster_id int OUTPUT,@is_survivor char(1) OUTPUT',@parm_mdm_entity_id,@cluster_id OUTPUT,@is_survivor OUTPUT
			   
			  SET @strSql = 'SELECT @active_contributors = count(1) FROM mdm_'+@parm_entity_type+'_tt
                              WHERE mdm_entity_cluster_id = @cluster_id
                                AND mdm_survivor = ''N''
                                AND mdm_valid_from_dttm <= @target_dttm
                                AND (mdm_valid_to_dttm IS NULL OR mdm_valid_to_dttm > @target_dttm)';
              EXECUTE sp_executesql @strSql,N'@cluster_id int,@target_dttm DATETIME,@active_contributors int OUTPUT',@cluster_id,@target_dttm,@active_contributors OUTPUT
			  
              IF (@is_survivor = 'Y') OR (@active_contributors <= 1) 
                 BEGIN                        
                     EXEC stwd_entityCluster_r @cluster_id,@parm_entity_type,@parm_userid,@target_dttm                       
                 END
              ELSE
                 BEGIN
                     EXEC stwd_entity_r @parm_mdm_entity_id,@parm_entity_type,@parm_userid,@target_dttm                      
                 END
              
              SET @actual_retire_date = @target_dttm                  
		  END 
	   COMMIT TRAN
	   SELECT @actual_retire_date AS retire_date
	END TRY
	BEGIN CATCH
		ROLLBACK TRAN

		DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									' : '+@errorSeverityString+
									' : '+@mdm_error_status+
									' : '+@errorStateString
		-- Log Error
		--NOTE Do we log old or new id?
		EXEC [mdm_log_error] null,null,@parm_mdm_entity_id,'mdm_entity_id','mdm_entity_r',@mdm_error_status,@mdm_error_messages,@errorDate

		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
	END CATCH

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_findLatestSurvivor]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_findLatestSurvivor]
GO

CREATE PROCEDURE [mdm_findLatestSurvivor] (@parm_mdm_entity_cluster_id INT, @parm_entity_type VARCHAR(23))
  AS
  
  DECLARE @sqlStr nvarchar(1000);
  DECLARE @tableName varchar(30);
  BEGIN
  BEGIN TRY
    SET @tableName = 'MDM_'+@parm_entity_type+'_TT'
    
    SET @sqlStr = 'select mdm_entity_id as entity_id from 
                    (select mdm_entity_id,mdm_valid_to_dttm,
                            max(isnull(mdm_valid_to_dttm,cast(''12/31/9999'' as datetime))) 
                              over (partition by mdm_entity_cluster_id) max_valid_to_dttm 
                       from '+@tableName+' 
                      where mdm_survivor=''Y''
                        and mdm_entity_cluster_id=@cluster_id) t 
                    where mdm_valid_to_dttm is null or mdm_valid_to_dttm=max_valid_to_dttm';
    EXECUTE sp_executesql @sqlStr,N'@cluster_id int',@parm_mdm_entity_cluster_id                  
  END TRY
  BEGIN CATCH
    DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
    SET @error_messages=ERROR_MESSAGE();
    SET @errorSeverity=ERROR_SEVERITY();
    SET @errorState=ERROR_STATE();
    SET @errorStatus=ERROR_NUMBER();
    SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
    set @dttm=getdate()
    SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
    EXEC [mdm_log_error] @tableName,null,@parm_mdm_entity_cluster_id,null,'mdm_findLatestSurvivor',@strErrorStatus,@mdm_error_messages,@dttm
    RAISERROR(@error_messages, @errorSeverity, @errorState)
  END CATCH
  END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_search_entity]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_search_entity]
GO

CREATE PROCEDURE [mdm_search_entity]
	@entityType varchar(50),
	@showRetired char(1),
	@roles VARCHAR(500),
	@rowlimit int,
    @whereClause varchar(2000),
    @orderBy varchar(30),
	@ascending char(1)
  AS
  	DECLARE @sqlString nvarchar(max)
  	DECLARE @strActiveClause nvarchar(200)
    DECLARE @strRowlimit NVARCHAR(10)
    DECLARE @tt_columns NVARCHAR(MAX)
    DECLARE @entityTypeId int
    DECLARE @dataMaskingSql NVARCHAR(2000);
    DECLARE @get_subclasses CURSOR;
    DECLARE @isAbstract CHAR(1);
    DECLARE @noConcreteSubclass CHAR(1);
    DECLARE @retiredWhereClause NVARCHAR(200);
    DECLARE @retiredSurvivorClause NVARCHAR(200);
    DECLARE @ascendingClause NVARCHAR(10);
    DECLARE @label_id int;
    DECLARE @attr_count INT;
    DECLARE @attr_id INT;
    DECLARE @attr_name  VARCHAR(50);
    DECLARE @raw_columns NVARCHAR(MAX);
    DECLARE @j INT;

    BEGIN
	SET NOCOUNT ON
	
    SELECT @entityTypeId = mdm_entity_type_id, @isAbstract = mdm_abstract FROM mdm_entity_types WHERE lower(mdm_entity_type) = lower(@entityType)
    
    SET @get_subclasses=CURSOR FAST_FORWARD FOR 
      WITH tree (mdm_entity_type_id, mdm_abstract, mdm_parent_id) AS 
      (SELECT et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id
      FROM mdm_entity_types et 
      WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
        AND et.mdm_entity_type_id = @entityTypeId
      UNION ALL 
      SELECT et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id 
      FROM mdm_entity_types et, tree t 
      WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
        AND et.mdm_parent_id = t.mdm_entity_type_id
        AND et.mdm_entity_type_id > 0)
      SELECT MIN(mdm_abstract) FROM tree WHERE mdm_entity_type_id <> @entityTypeId;
                       
    IF (@isAbstract = 'Y') 
      BEGIN
        OPEN @get_subclasses;
          FETCH NEXT FROM @get_subclasses INTO @noConcreteSubclass;
        CLOSE @get_subclasses;
        
        IF (@noConcreteSubclass = 'Y') OR (@noConcreteSubclass IS NULL)
         BEGIN
          SET @sqlString = 'This Entity Type has no concrete subclasses to search'
          RAISERROR(@sqlString, 11,1);
          RETURN;
         END        
      END
    
    CREATE TABLE #colStr (colStrValue nvarchar(max));
    
    INSERT INTO #colStr    
    EXEC [mdm_getDecryptTtCols] @entityTypeId    
    SELECT @tt_columns = colStrValue FROM #colStr  
    DROP TABLE #colStr;
    
    CREATE TABLE #rawTbl (id int identity(1,1),colStrId int,colStrValue varchar(30));
    INSERT INTO #rawTbl (colStrId,colStrValue)
    EXEC [mdm_getAttrs] @entityType,'D'
    SELECT @attr_count = COUNT(1) FROM #rawTbl
    SET @raw_columns = '' 
    SET @j = 1
    WHILE (@j <= @attr_count) BEGIN
       SELECT @attr_name = colStrValue,@attr_id=colStrId FROM #rawTbl WHERE id = @j
       IF (@attr_id > 0)
         SET @raw_columns = @raw_columns+','+@attr_name
       SET @j = @j+1
    END
    DROP TABLE #rawTbl;

    -- anything with a masked label attribute needs to be removed from the search
    WITH ancestors (mdm_entity_type, mdm_entity_type_id, mdm_label_attr_id, mdm_abstract, mdm_parent_id) AS 
      (SELECT et.mdm_entity_type, et.mdm_entity_type_id, et.mdm_label_attr_id, et.mdm_abstract, et.mdm_parent_id
       FROM mdm_entity_types et 
       WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
         AND et.MDM_ENTITY_TYPE_ID = @entityTypeId 
       UNION ALL
       SELECT et.mdm_entity_type, et.mdm_entity_type_id, et.mdm_label_attr_id,et.mdm_abstract, et.mdm_parent_id 
       FROM mdm_entity_types et, ancestors t 
       WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null)
         AND et.mdm_valid_from_dttm <= GETDATE()
         AND et.mdm_entity_type_id = t.mdm_parent_id)
    select @label_id=mdm_label_attr_id 
    from ancestors
    where mdm_label_attr_id is not null; 
    
    SET @dataMaskingSql = 
      'select x.mdm_entity_type_id from (select er.mdm_entity_type_id from mdm_entity_type_access er 
        where lower(er.mdm_group) in ('+lower(@roles)+')  
          and er.mdm_active = ''Y''
        union 
        select et.mdm_entity_type_id from mdm_entity_types et 
        where et.mdm_entity_type_id not in 
          (select mdm_entity_type_id from mdm_entity_type_access where mdm_active = ''Y'') ) x
      inner join mdm_entity_types et on et.mdm_entity_type_id=x.mdm_entity_type_id and et.mdm_valid_to_dttm is null
      where x.mdm_entity_type_id not in (select mdm_entity_type_id from MDM_ATTR_ACCESS at where mdm_attr_id='+CONVERT(VARCHAR,@label_id)+' and lower(at.MDM_GROUP) not in ('+lower(@roles)+') and at.MDM_ACTIVE=''Y'')'

    IF (@rowlimit = null)
      set @strRowLimit = '100'
    ELSE
      set @strRowLimit=CONVERT(NVARCHAR,@rowLimit)          
    
    IF (@showRetired='N')
      begin
        set @retiredWhereClause='  and mdm_valid_to_dttm is null ';
        set @retiredSurvivorClause=' mdm_valid_to_dttm is null ';
      end
    ELSE
      begin
        set @retiredWhereClause=' ';
        set @retiredSurvivorClause=' (mdm_valid_to_dttm is null or mdm_valid_to_dttm = max_valid_to_dttm) ';
      end
      
    IF (@ascending='Y')
      begin
        set @ascendingClause=' asc ';
      end
    ELSE
      begin
        set @ascendingClause=' desc ';
      end

    set @sqlString='with searchresults as (
      select 
        row_number() over (order by mdm_entity_cluster_id) as rownumber, 
        mdm_entity_type_id,mdm_entity_cluster_id,mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,mdm_valid_from_dttm,mdm_valid_to_dttm'+@raw_columns+' from
        (
        select max_valid_to_dttm,mdm_entity_type_id,mdm_entity_cluster_id,mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,mdm_valid_from_dttm,mdm_valid_to_dttm'+@tt_columns+'
        from mdm_'+@entityType+'_sv 
        where mdm_entity_cluster_id in (select distinct mdm_entity_cluster_id from mdm_'+@entityType+'_sv where '+@whereClause+@retiredWhereClause+')
          and mdm_entity_type_id in ('+@dataMaskingSql+')
        ) x
        where '+@retiredSurvivorClause+' and mdm_src_sys_id=1)
      select top '+@strRowLimit+' mdm_entity_type_id,mdm_entity_cluster_id,mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,mdm_valid_from_dttm,mdm_valid_to_dttm'+@raw_columns+' from searchresults
        order by '+@orderBy+@ascendingClause
  BEGIN TRY
    OPEN SYMMETRIC KEY qMDMKey DECRYPTION BY CERTIFICATE qMDMCer
    EXECUTE sp_executesql @sqlString
    CLOSE SYMMETRIC KEY qMDMKey
  END TRY
  BEGIN CATCH
    DECLARE @searchView VARCHAR(50)
    SELECT @searchView = 'MDM_'+UPPER(@entityType)+'SV'
    DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
    SET @error_messages=ERROR_MESSAGE();
    SET @errorSeverity=ERROR_SEVERITY();
    SET @errorState=ERROR_STATE();
    SET @errorStatus=ERROR_NUMBER();
    SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
    set @dttm=getdate()
    SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
    EXEC [mdm_log_error] @searchView,null,0,null,'mdm_search_entity',@strErrorStatus,@mdm_error_messages,@dttm
    RAISERROR(@error_messages, @errorSeverity, @errorState)
  END CATCH
  END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_search_relationship]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_search_relationship]
GO

CREATE PROCEDURE [mdm_search_relationship]
	@clusterId int,
	@relType varchar(50),
	@showRelated char(1),
	@showInverse char(1),
	@roles VARCHAR(500),
	@rowlimit int,
    @whereClause varchar(2000)
  AS
  	DECLARE @searchSql nvarchar(max)
  	DECLARE @matchSql nvarchar(4000);
  	DECLARE @authoredRelSql nvarchar(400);
    DECLARE @retiredRelSql nvarchar(400);
    DECLARE @dataMaskingSql NVARCHAR(2000);
    DECLARE @strRowLimit nvarchar(10)
    DECLARE @srcEntityType nvarchar(23)
    DECLARE @tgtEntityType nvarchar(23)
    DECLARE @srcEntityTypeId int
    DECLARE @tgtEntityTypeId int
    DECLARE @srcIsAbstract char(1)
    DECLARE @tgtIsAbstract char(1)
    DECLARE @isAbstract char(1)
    DECLARE @searchEntityType nvarchar(23)
    DECLARE @searchTargetType nvarchar(23)
    DECLARE @searchEntityTypeid int
    DECLARE @relTypeId int
    DECLARE @noConcreteSubclass CHAR(1);
    DECLARE @tt_columns nvarchar(max)
    DECLARE @conditionId int
    DECLARE @prevConditionId int
    DECLARE @srcAttr nvarchar(30)
    DECLARE @tgtAttr nvarchar(30)
    DECLARE @subjectAttr nvarchar(30)
    DECLARE @objectAttr nvarchar(30)
    DECLARE @get_conditions CURSOR;
    
  BEGIN
    SET NOCOUNT ON

    IF (@rowlimit = null)
      set @strRowLimit = '100'
    ELSE
      set @strRowLimit=CONVERT(NVARCHAR,@rowLimit)          

    -- Turn the relationship type into entity type information
    select @relTypeId=rt.mdm_rel_type_id,@srcEntityType=fr_et.mdm_entity_type,@srcEntityTypeId=rt.mdm_from_entity_type_id,@srcIsAbstract=fr_et.mdm_abstract,
      @tgtEntityType=to_et.mdm_entity_type,@tgtEntityTypeId=rt.mdm_to_entity_type_id,@tgtIsAbstract=to_et.mdm_abstract
    from mdm_entity_rel_types rt
    inner join mdm_entity_types fr_et on rt.mdm_from_entity_type_id=fr_et.mdm_entity_type_id
    inner join mdm_entity_types to_et on rt.mdm_to_entity_type_id=to_et.mdm_entity_type_id
    where lower(@relType)=lower(rt.mdm_rel_type)
      and rt.mdm_valid_from_dttm <=getdate() and (rt.mdm_valid_to_dttm is null or rt.mdm_valid_to_dttm > getdate());

    IF (@showInverse = 'N') BEGIN
      set @searchEntityType=@tgtEntityType;
      set @searchTargetType=@srcEntityType;
      set @searchEntityTypeId=@tgtEntityTypeId;
      set @isAbstract=@tgtIsAbstract;
      set @authoredRelSql='select mdm_to_cluster_id from mdm_entity_rel where mdm_entity_rel_type_id='
        +CONVERT(NVARCHAR,@relTypeId)+' and mdm_from_cluster_id='+CONVERT(NVARCHAR,@clusterId) 
        +' and (mdm_valid_to_dttm is null or mdm_valid_to_dttm > getdate())';
      set @retiredRelSql='select mdm_to_cluster_id from mdm_entity_rel where mdm_entity_rel_type_id='
        +CONVERT(NVARCHAR,@relTypeId)+' and mdm_from_cluster_id='+CONVERT(NVARCHAR,@clusterId)
        +' and mdm_valid_to_dttm <= getdate()';
    END
    ELSE BEGIN
      set @searchEntityType=@srcEntityType;
      set @searchTargetType=@tgtEntityType;
      set @searchEntityTypeId=@srcEntityTypeId;
      set @isAbstract=@srcIsAbstract;
      set @authoredRelSql='select mdm_from_cluster_id from mdm_entity_rel where mdm_entity_rel_type_id='
        +CONVERT(NVARCHAR,@relTypeId)+' and mdm_to_cluster_id='+CONVERT(NVARCHAR,@clusterId)
        +' and (mdm_valid_to_dttm is null or mdm_valid_to_dttm > getdate())';
      set @retiredRelSql='select mdm_from_cluster_id from mdm_entity_rel where mdm_entity_rel_type_id='
        +CONVERT(NVARCHAR,@relTypeId)+' and mdm_to_cluster_id='+CONVERT(NVARCHAR,@clusterId)
        +' and mdm_valid_to_dttm <= getdate()';
    END
                           
    IF (@isAbstract = 'Y') BEGIN
      WITH tree (mdm_entity_type_id, mdm_abstract, mdm_parent_id) AS 
      (SELECT et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id
       FROM mdm_entity_types et 
       WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
         AND et.mdm_entity_type_id = @searchEntityTypeId
       UNION ALL 
       SELECT et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id 
       FROM mdm_entity_types et, tree t 
       WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
         AND et.mdm_parent_id = t.mdm_entity_type_id
         AND et.mdm_entity_type_id > 0)
      SELECT @noConcreteSubclass=MIN(mdm_abstract) FROM tree WHERE mdm_entity_type_id <> @searchEntityTypeId;
        
      IF (@noConcreteSubclass = 'Y') OR (@noConcreteSubclass IS NULL) BEGIN
        RAISERROR('This Entity Type has no concrete subclasses to search', 11,1);
        RETURN;
      END        
    END
    
    IF (@showRelated='Y') BEGIN
	  set @get_conditions=CURSOR FAST_FORWARD FOR
	    select c.mdm_entity_rel_condition_id,fr_attr.mdm_attr_name as from_attr,to_attr.mdm_attr_name as to_attr
        from mdm_entity_rel_conditions c
        inner join mdm_attrs fr_attr on c.mdm_from_attr_id=fr_attr.mdm_attr_id
        inner join mdm_attrs to_attr on c.mdm_to_attr_id=to_attr.mdm_attr_id
        where mdm_entity_rel_type_id=@relTypeId
        order by mdm_entity_rel_condition_id;
      set @matchSQL='';
      OPEN @get_conditions;
      FETCH NEXT FROM @get_conditions into @conditionId,@srcAttr,@tgtAttr;
      WHILE @@FETCH_STATUS = 0 BEGIN
        IF (@showInverse = 'N') BEGIN
          set @subjectAttr=@tgtAttr;
          set @objectAttr=@srcAttr;
        END
        ELSE BEGIN
          set @subjectAttr=@srcAttr;
          set @objectAttr=@tgtAttr;
        END;
        IF (@prevConditionId is null)
          set @matchSql='';
        ELSE BEGIN
          IF (@prevConditionId <> @conditionId)
            set @matchSql=@matchSql+' OR ';
          ELSE
            set @matchSql=@matchSql+' AND ';
        END;
        set @matchSql = @matchSql+@subjectAttr+' in (select distinct '+@objectAttr
          +' from mdm_'+@searchTargetType+'_sv where mdm_entity_cluster_id='+ CONVERT(NVARCHAR,@clusterId)
          +' and mdm_src_sys_id > 1 and (mdm_valid_to_dttm is null  or mdm_valid_to_dttm > getdate()))';
        set @prevConditionId = @conditionId;
        FETCH NEXT FROM @get_conditions into @conditionId,@srcAttr,@tgtAttr;
      END
      -- no conditions? then we can't return any results. Make the where clause shut everything down.
      IF (len(ltrim(rtrim(@matchSql))) = 0)
        set @matchSql='1=2';
      CLOSE @get_conditions;
      DEALLOCATE @get_conditions;
    END
    ELSE BEGIN
      set @authoredRelSql='select 0 as mdm_from_cluster_id from dual where 1=2';
      set @retiredRelSql='select 0 as mdm_from_cluster_id from dual where 1=2';
      -- We might not have a whereclause. If so, default to a tautology
      IF ((@whereClause IS NULL) OR (len(ltrim(rtrim(@whereClause)))) = 0)
        set @matchSql='1=1';
      ELSE
        set @matchSql=@whereClause;
    END

    -- Get the columns for the entity type
    CREATE TABLE #colStr (colStrValue nvarchar(max));   
    INSERT INTO #colStr    
    EXEC [mdm_getDecryptTtCols] @searchEntityTypeId    
    SELECT @tt_columns = colStrValue FROM #colStr  
    DROP TABLE #colStr
    
    SET @dataMaskingSql = 'select er.mdm_entity_type_id from mdm_entity_type_access er 
      where lower(er.mdm_group) in ('+lower(@roles)+')  
        and er.mdm_active = ''Y''
      union 
      select et.mdm_entity_type_id from mdm_entity_types et 
      where et.mdm_entity_type_id not in 
        (
        select mdm_entity_type_id from mdm_entity_type_access 
        where mdm_active = ''Y''
        )'

    set @searchSql='select top '+@strRowLimit+' mdm_entity_type_id,mdm_entity_cluster_id,mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,mdm_valid_from_dttm,mdm_valid_to_dttm'+@tt_columns+' from
        (
        select max_valid_to_dttm,mdm_entity_type_id,mdm_entity_cluster_id,mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,mdm_valid_from_dttm,mdm_valid_to_dttm'+@tt_columns+'
        from mdm_'+@searchEntityType+'_sv 
        where mdm_entity_cluster_id in 
         (select distinct mdm_entity_cluster_id from (select distinct mdm_entity_cluster_id from mdm_'+@searchEntityType+'_sv 
          where (mdm_valid_to_dttm is null or mdm_valid_to_dttm >getdate()) and '+@matchSql+' union '+@authoredRelSql+') t )
            and mdm_entity_cluster_id not in ('+ @retiredRelSql+')
            and mdm_entity_type_id in ('+@dataMaskingSql+')
            and mdm_valid_from_dttm < getdate() 
        ) x
      where (mdm_valid_to_dttm is null or mdm_valid_to_dttm > getdate()) 
        and mdm_src_sys_id=1 
      order by mdm_entity_cluster_id'
		     
  BEGIN TRY
    OPEN SYMMETRIC KEY qMDMKey DECRYPTION BY CERTIFICATE qMDMCer
    EXECUTE sp_executesql @searchSql
    CLOSE SYMMETRIC KEY qMDMKey
  END TRY
  BEGIN CATCH
    DECLARE @transposeView VARCHAR(50)
    SELECT @transposeView = 'MDM_'+UPPER(@searchEntityType)+'SV'
    DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
    SET @error_messages=ERROR_MESSAGE();
    SET @errorSeverity=ERROR_SEVERITY();
    SET @errorState=ERROR_STATE();
    SET @errorStatus=ERROR_NUMBER();
    SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
    set @dttm=getdate()
    SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
    EXEC [mdm_log_error] @transposeView,null,0,null,'mdm_search_relationship',@strErrorStatus,@mdm_error_messages,@dttm
    RAISERROR(@error_messages, @errorSeverity, @errorState)
  END CATCH
  END
GO

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_add_relationship]') AND type in (N'P', N'PC'))
 drop procedure mdm_add_relationship
GO

CREATE PROCEDURE mdm_add_relationship(@relTypeId INTEGER, @fromClusterId INTEGER, @toClusterId INTEGER)
AS
  DECLARE @active_dttm DATETIME;
BEGIN
  SET NOCOUNT ON	
  BEGIN TRY	
    insert into mdm_entity_rel (mdm_entity_rel_type_id,mdm_from_cluster_id,mdm_to_cluster_id)
	values (@relTypeId,@fromClusterId,@toClusterId)
  SELECT mdm_valid_from_dttm FROM mdm_entity_rel where MDM_ENTITY_REL_TYPE_ID=@relTypeId 
    and MDM_FROM_CLUSTER_ID=@fromClusterId and MDM_TO_CLUSTER_ID=@toClusterId;
  END TRY
  BEGIN CATCH
    DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
    SET @errorStatus=ERROR_NUMBER();
    if (@errorStatus=2627) -- key violation
      BEGIN
	    -- If it was retired, unretire it.
        update mdm_entity_rel set mdm_valid_to_dttm=null,mdm_valid_from_dttm=getdate() where MDM_ENTITY_REL_TYPE_ID=@relTypeId
          and MDM_FROM_CLUSTER_ID=@fromClusterId and MDM_TO_CLUSTER_ID=@toClusterId
          and MDM_VALID_TO_DTTM <=getdate();
        SELECT mdm_valid_from_dttm FROM mdm_entity_rel where MDM_ENTITY_REL_TYPE_ID=@relTypeId 
          and MDM_FROM_CLUSTER_ID=@fromClusterId and MDM_TO_CLUSTER_ID=@toClusterId;
        return;
      END
    SET @error_messages=ERROR_MESSAGE();
    SET @errorSeverity=ERROR_SEVERITY();
    SET @errorState=ERROR_STATE();
    SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
    set @dttm=getdate()
    SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
    EXEC [mdm_log_error] 'MDM_ENTITY_REL',null,@relTypeId,null,'mdm_addRelationship',@strErrorStatus,@mdm_error_messages,@dttm
    RAISERROR(@error_messages, @errorSeverity, @errorState)
  END CATCH
END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_retire_relationship]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_retire_relationship]
GO

CREATE PROCEDURE [mdm_retire_relationship](@relTypeId INTEGER, @fromClusterId INTEGER, @toClusterId INTEGER)
AS
  DECLARE @retire_dttm DATETIME;
BEGIN
  SET NOCOUNT ON
  set @retire_dttm=GETDATE()
      
  BEGIN TRY
    merge into mdm_entity_rel r
      using (select @relTypeId as relTypeId,@fromClusterId as fromClusterId,@toClusterId as toClusterId from dual) contr 
      on (r.MDM_ENTITY_REL_TYPE_ID=contr.relTypeId and r.MDM_FROM_CLUSTER_ID=contr.fromClusterId and r.MDM_TO_CLUSTER_ID=contr.toClusterId) 
      when matched then update set r.mdm_valid_to_dttm=@retire_dttm
      when not matched then 
        insert (MDM_ENTITY_REL_TYPE_ID,MDM_FROM_CLUSTER_ID,MDM_TO_CLUSTER_ID,MDM_VALID_FROM_DTTM,MDM_VALID_TO_DTTM)
        values (contr.relTypeId,contr.fromClusterId,contr.toClusterId,getdate(),@retire_dttm);
    SELECT mdm_valid_to_dttm FROM mdm_entity_rel where MDM_ENTITY_REL_TYPE_ID=@relTypeId 
      and MDM_FROM_CLUSTER_ID=@fromClusterId and MDM_TO_CLUSTER_ID=@toClusterId;
  END TRY
  BEGIN CATCH
    DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
    SET @errorStatus=ERROR_NUMBER();
    SET @error_messages=ERROR_MESSAGE();
    SET @errorSeverity=ERROR_SEVERITY();
    SET @errorState=ERROR_STATE();
    SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
    set @dttm=getdate()
    SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
    EXEC [mdm_log_error] 'MDM_ENTITY_REL',null,@relTypeId,null,'mdm_retireRelationship',@strErrorStatus,@mdm_error_messages,@dttm
    RAISERROR(@error_messages, @errorSeverity, @errorState)
  END CATCH
END

GO


IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[stwd_updateClusterMapping]') AND type in (N'P', N'PC'))
 drop procedure stwd_updateClusterMapping
GO

CREATE PROCEDURE stwd_updateClusterMapping (@parm_cluster_id      INTEGER,
					    @parm_new_cluster_id  INTEGER) AS
  BEGIN
    BEGIN TRY
     
     UPDATE mdm_cluster_mapping
        SET current_cluster_id = @parm_new_cluster_id
      WHERE original_cluster_id = @parm_cluster_id;
      
    END TRY
    
    BEGIN CATCH     
          DECLARE @error_messages VARCHAR(2000)
	  DECLARE @errorSeverity INT
	  DECLARE @errorState  INT
	  DECLARE @errorStatus INT
	  DECLARE @errorProc  VARCHAR(150)
	  DECLARE @errorDate DATETIME

	  DECLARE @mdm_error_status  VARCHAR(10)
	  DECLARE @mdm_error_messages VARCHAR(4000)
	  DECLARE @errorSeverityString VARCHAR(100)
	  DECLARE @errorStateString VARCHAR(100)

	  SELECT @errorStatus = ERROR_NUMBER()
	  SELECT @error_messages = ERROR_MESSAGE()
	  SELECT @errorSeverity = ERROR_SEVERITY()
	  SELECT @errorState  = ERROR_STATE()
	  SELECT @errorProc = ERROR_PROCEDURE()
	  SELECT @errorDate = getDate()

	  SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	  SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	  SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	  SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	   -- Log Error
	   EXEC [mdm_log_error] 'MDM_CLUSTER_MAPPING',null,0,null,'stwd_updateClusterMapping',@mdm_error_status,@mdm_error_messages,@errorDate
	   -- Raise Error
	   RAISERROR (@error_messages, @errorSeverity, @errorState)
	END CATCH
  END
GO

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[MDM_START_PROCESS_CONTROL]') AND type in (N'P', N'PC'))
 DROP PROCEDURE MDM_START_PROCESS_CONTROL
GO

CREATE PROCEDURE [mdm_start_process_control]
(	
	@parm_staging_id           INT,
	@parm_process_name         VARCHAR(50),
	@parm_entity_type_id	   INT
)
AS
    DECLARE @timestamp   datetime;
	SET NOCOUNT ON
	BEGIN TRY	
	  
	SELECT @timestamp = GETDATE()
	 
	BEGIN TRAN
		INSERT INTO [MDM_PROCESS_CONTROL]
			   (
			    mdm_staging_id
			   ,process_name
			   ,mdm_entity_type_id
			   ,process_running
			   ,process_start_dttm	)		   
		 VALUES
			   (
				@parm_staging_id
			   ,@parm_process_name
			   ,@parm_entity_type_id
			   ,'Y'
			   ,@timestamp)
		
	COMMIT TRAN
		
	   SELECT process_id 
		 FROM mdm_process_control
		WHERE process_name = @parm_process_name
		  AND mdm_entity_type_id = @parm_entity_type_id
		  AND mdm_staging_id = @parm_staging_id
		  AND process_running = 'Y'
		  AND process_start_dttm = @timestamp;
	     
	END TRY
	BEGIN CATCH
		ROLLBACK TRAN

		DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									' : '+@errorSeverityString+
									' : '+@mdm_error_status+
									' : '+@errorStateString
		-- Log Error
		EXEC [mdm_log_error] 'MDM_PROCESS_CONTROL',null,0,null,'mdm_start_process_control',@mdm_error_status,@mdm_error_messages,@errorDate

		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
	END CATCH

GO

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[MDM_END_PROCESS_CONTROL]') AND type in (N'P', N'PC'))
 DROP PROCEDURE MDM_END_PROCESS_CONTROL
GO


CREATE PROCEDURE [mdm_end_process_control]
(	
	@parm_process_id       INT,
	@parm_process_success  CHAR(1)
)
AS	
	DECLARE @end_dttm DATETIME
    SET NOCOUNT ON
	BEGIN TRY   		
	  BEGIN TRAN
		UPDATE mdm_process_control
		   SET process_running = 'N',
		       process_success = @parm_process_success,
		       process_end_dttm = GETDATE()
		 WHERE process_id = @parm_process_id
      COMMIT TRAN
	END TRY
	BEGIN CATCH
		ROLLBACK TRAN
		
		DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									' : '+@errorSeverityString+
									' : '+@mdm_error_status+
									' : '+@errorStateString
		-- Log Error
		EXEC [mdm_log_error] 'MDM_PROCESS_CONTROL',null,0,null,'mdm_end_process_control',@mdm_error_status,@mdm_error_messages,@errorDate

		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
	END CATCH
GO

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[stwd_addMemberHistory]') AND type in (N'P', N'PC'))
drop procedure stwd_addMemberHistory
GO

CREATE PROCEDURE stwd_addMemberHistory (@parm_old_cluster_id      INT,
										@parm_entity_id  INTEGER) AS

  BEGIN TRY

     INSERT INTO mdm_cluster_member_history
        (mdm_entity_cluster_id,
         mdm_entity_id,
         mdm_modified_dttm)
     VALUES
        (@parm_old_cluster_id,
         @parm_entity_id,
         getdate());
     
  END TRY

	BEGIN CATCH
		ROLLBACK TRAN

		DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									' : '+@errorSeverityString+
									' : '+@mdm_error_status+
									' : '+@errorStateString
		-- Log Error
		--NOTE Do we log old or new id?
		EXEC [mdm_log_error] 'MDM_CLUSTER_MEMBER_HISTORY',null,@parm_entity_id,'mdm_entity_id','stwd_addMemberHistory',@mdm_error_status,@mdm_error_messages,@errorDate

		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
	END CATCH

GO


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_addUpdate_forcedCluster]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_addUpdate_forcedCluster]
GO

CREATE PROCEDURE [mdm_addUpdate_forcedCluster](@parm_ori_cluster_id   INTEGER,
					     @parm_curr_cluster_id  INTEGER,
					     @parm_ssid             INTEGER,
					     @parm_ssrid            VARCHAR(200),
                                             @parm_entity_type_id   INTEGER) AS

  DECLARE @record_exists      INTEGER;
  DECLARE @old_curr_cluster   INTEGER;  
  DECLARE @cluster_id_exists  INTEGER;
  
  SET NOCOUNT ON
  
    BEGIN TRY
		SELECT @record_exists = count(1)
		  FROM mdm_forced_clusters
		 WHERE src_sys_id = @parm_ssid
		   AND src_sys_rec_id = @parm_ssrid
		   AND entity_type_id = @parm_entity_type_id;

		IF (@record_exists > 0) 
		  BEGIN	           
			   SELECT @old_curr_cluster = current_cluster_id
				 FROM mdm_forced_clusters
				WHERE src_sys_id = @parm_ssid
				  AND src_sys_rec_id = @parm_ssrid
				  AND entity_type_id = @parm_entity_type_id;      
				      
			   UPDATE mdm_forced_clusters
				  SET current_cluster_id = @parm_curr_cluster_id
				WHERE src_sys_id = @parm_ssid
				  AND src_sys_rec_id = @parm_ssrid
				  AND entity_type_id = @parm_entity_type_id;
			  
			   SELECT @cluster_id_exists = count(1)
			     FROM mdm_forced_clusters
			    WHERE current_cluster_id = @old_curr_cluster;
			    
			   IF (@cluster_id_exists = 0)
			     BEGIN
			       UPDATE mdm_entity_clusters 
					  SET mdm_cluster_type = 'N'
				    WHERE mdm_entity_cluster_id = @old_curr_cluster;
			     END
	      END    
		ELSE
	      BEGIN
			   INSERT INTO mdm_forced_clusters
				  (original_cluster_id,
				   src_sys_id,
				   src_sys_rec_id,
				   entity_type_id,
				   current_cluster_id)
			   VALUES
				  (@parm_ori_cluster_id,
				   @parm_ssid,
				   @parm_ssrid,
				   @parm_entity_type_id,
				   @parm_curr_cluster_id);
	           
		  END
        
        UPDATE mdm_entity_clusters
           SET mdm_cluster_type = 'F'
         WHERE mdm_entity_cluster_id = @parm_curr_cluster_id;
      
    END TRY
    BEGIN CATCH     
       DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	   SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	   -- Log Error
	   EXEC [mdm_log_error] 'MDM_FORCED_CLUSTERS',null,0,null,'mdm_addUpdate_forcedCluster',@mdm_error_status,@mdm_error_messages,@errorDate
	   -- Raise Error
	   RAISERROR (@error_messages, @errorSeverity, @errorState)
	END CATCH  
	
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_get_matchingRows]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_get_matchingRows]
GO

CREATE PROCEDURE [mdm_get_matchingRows](@parm_staging_id INTEGER,@parm_entity_type VARCHAR(23)) 
AS
DECLARE @sql_str NVARCHAR(MAX);
DECLARE @conditionStr NVARCHAR(MAX);
DECLARE @conditionId INT;
DECLARE @isFirstCondition INT;
DECLARE @isNewCondition INT;
DECLARE @prevCondition INT;
DECLARE @tt_columns NVARCHAR(MAX);
DECLARE @attr_name NVARCHAR(50);
DECLARE @encrypt_flag CHAR(1);
DECLARE @parent_id INT;
DECLARE @get_attrs CURSOR;
DECLARE @get_clusterConditions CURSOR;
DECLARE @entityTypeId INT;
DECLARE @entityCc_tbl VARCHAR(30);

BEGIN
  SET NOCOUNT ON
  BEGIN TRY
    SET @entityCc_tbl = 'MDM_BULK_CC_'+CONVERT(VARCHAR,@parm_staging_id);
    -- Get entity type id
    SELECT @entityTypeId = mdm_entity_type_id FROM mdm_entity_types WHERE LOWER(mdm_entity_type) = LOWER(@parm_entity_type)
    
    -- Build columns string
    SET @get_attrs=CURSOR FAST_FORWARD FOR WITH tree (mdm_entity_type_id, mdm_parent_id) AS 
     (SELECT et.mdm_entity_type_id, et.mdm_parent_id
      FROM mdm_entity_types et 
      WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
        AND lower(et.mdm_entity_type) = lower(@parm_entity_type)
      UNION ALL 
      SELECT et.mdm_entity_type_id, et.mdm_parent_id 
      FROM mdm_entity_types et, tree t 
      WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
        AND et.MDM_ENTITY_TYPE_ID = t.mdm_parent_id)    
      SELECT mdm_attr_name,mdm_encrypt_flag,mdm_parent_attr_id FROM mdm_attrs
      WHERE mdm_entity_type_id IN (SELECT mdm_entity_type_id FROM tree)
        AND mdm_valid_from_dttm < GETDATE() and (mdm_valid_to_dttm is null or mdm_valid_to_dttm > GETDATE())
        AND mdm_attr_id > 0;
    
    SET @tt_columns = '';
    OPEN @get_attrs;
    FETCH NEXT FROM @get_attrs INTO @attr_name,@encrypt_flag,@parent_id;
    WHILE @@FETCH_STATUS = 0 BEGIN
      IF (@encrypt_flag = 'Y' and @parent_id IS NULL)
          SET @tt_columns = @tt_columns+',CONVERT(varchar,DECRYPTBYKEY('+@attr_name+')) as '+@attr_name;
      ELSE 
          SET @tt_columns = @tt_columns+','+@attr_name;
      FETCH NEXT FROM @get_attrs INTO @attr_name,@encrypt_flag,@parent_id;        
    END  
    CLOSE @get_attrs;
    DEALLOCATE @get_attrs; 
    
    -- Create temp table
    CREATE TABLE #cluster_tbl (cluster_id INT);
    CREATE INDEX idx_cls_id ON #cluster_tbl (cluster_id);
    
    -- Build cluster condition string
    SET @get_clusterConditions=CURSOR FAST_FORWARD FOR WITH tree (mdm_entity_type_id, mdm_parent_id) AS 
     (SELECT et.mdm_entity_type_id, et.mdm_parent_id
      FROM mdm_entity_types et 
      WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
        AND lower(et.mdm_entity_type) = lower(@parm_entity_type)
      UNION ALL 
      SELECT et.mdm_entity_type_id, et.mdm_parent_id 
      FROM mdm_entity_types et, tree t 
      WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
        AND et.MDM_ENTITY_TYPE_ID = t.mdm_parent_id)
      SELECT mdm_cluster_condition_id, mdm_attr_name
      FROM mdm_cluster_condition_vw
      WHERE mdm_entity_type_id IN (SELECT mdm_entity_type_id FROM tree)
      ORDER BY mdm_cluster_condition_id;
    
    SET @conditionStr = '';
    SET @isFirstCondition = 1;
    
    OPEN @get_clusterConditions;
    FETCH NEXT FROM @get_clusterConditions INTO @conditionId,@attr_name;
    WHILE @@FETCH_STATUS = 0 BEGIN 
      IF (@prevCondition = @conditionId) 
        set @isNewCondition=0;
      ELSE
        set @isNewCondition=1;
      
      IF (@isFirstCondition=1) 
        BEGIN
          set @isFirstCondition=0;
          set @conditionStr=@conditionStr+' SELECT ma.mdm_entity_cluster_id FROM mdm_'+@parm_entity_type+'_tt ma, '+@entityCc_tbl+' s WHERE ';
        END
      ELSE IF (@isNewCondition=1)
        set @conditionStr=@conditionStr+' UNION SELECT ma.mdm_entity_cluster_id FROM mdm_'+@parm_entity_type+'_tt ma, '+@entityCc_tbl+' s WHERE ';
      ELSE
        set @conditionStr=@conditionStr+' AND ';

      set @conditionStr=@conditionStr+'s.'+@attr_name+'=ma.'+@attr_name
      set @prevCondition=@conditionId;
      FETCH NEXT FROM @get_clusterConditions INTO @conditionId,@attr_name;
    END
    CLOSE @get_clusterConditions;
    DEALLOCATE @get_clusterConditions; 
    
   
    SET @sql_str = 'SELECT fv.original_cluster_id, fv.current_cluster_id,ma1.mdm_entity_type_id,ma1.mdm_entity_id,ma1.mdm_src_sys_id,
              ma1.mdm_src_sys_rec_id,ma1.mdm_language_id,ma1.mdm_valid_from_dttm,ma1.mdm_valid_to_dttm,ma1.mdm_entity_cluster_id,
              ma1.mdm_survivor'+@tt_columns+'
         FROM mdm_'+@parm_entity_type+'_ma ma1
         LEFT OUTER JOIN mdm_forced_clusters fv ON fv.src_sys_id = ma1.mdm_src_sys_id AND fv.src_sys_rec_id = ma1.mdm_src_sys_rec_id AND fv.entity_type_id = @entityTypeId
        WHERE mdm_entity_cluster_id IN (SELECT DISTINCT mdm_entity_cluster_id FROM ('+@conditionStr+') tmp)';
								              
    EXECUTE sp_executesql @sql_str,N'@entityTypeId int',@entityTypeId
  END TRY   
  BEGIN CATCH
    DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
    SET @error_messages=ERROR_MESSAGE();
    SET @errorSeverity=ERROR_SEVERITY();
    SET @errorState=ERROR_STATE();
    SET @errorStatus=ERROR_NUMBER();
    SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
    set @dttm=getdate()
    SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
	EXEC [mdm_log_error] 'MDM_GET_MATCHINGROWS',null,0,null,'mdm_get_matchingRows',@errorStatus,@error_messages,@dttm
    RAISERROR(@error_messages, @errorSeverity, @errorState)
  END CATCH
END

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_entity_golive]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_entity_golive]
GO

CREATE PROCEDURE [mdm_entity_golive] (@firstClusterId			INT,
										@lastClusterId			INT,
										@parm_staging_id		INT,
										@parm_entity_type_id	INT,
										@tt_columns				NVARCHAR(MAX),
										@parm_entity_type		VARCHAR(23),
										@parm_current_dttm		DATETIME,
										@parm_userid			NVARCHAR(513),
										@parm_process_type      VARCHAR(10),
										@parm_match_type        INT,
										@parm_comment           NVARCHAR(200)) 
AS
			
DECLARE @stgTbl		     VARCHAR(30);
DECLARE @entityTtTbl     VARCHAR(30);
DECLARE @entityStTbl     VARCHAR(30);
DECLARE @strSql				NVARCHAR(MAX);
DECLARE @strOldSurvivorSql	NVARCHAR(4000);
DECLARE @err_location		INT;
DECLARE @systimestamp		DATETIME;
DECLARE @errMessage         VARCHAR(100);
DECLARE @mtCount            INT;
DECLARE @k                  INT;
DECLARE @mt                 INT;
DECLARE @mt0                INT;
DECLARE @mt1                INT;
DECLARE @mt2                INT;
DECLARE @mt3                INT;
DECLARE @mt4                INT;
DECLARE @history_comment    NVARCHAR(200);
DECLARE @history_reason     CHAR(1);
DECLARE @mtClause           VARCHAR(30);
  
SET NOCOUNT ON
  
BEGIN
	BEGIN TRY
		
		BEGIN TRAN goLiveTran
		
			/**************************************************************************************************************************************************************************
			 Processing start
			**************************************************************************************************************************************************************************/
			
			SET @entityTtTbl = 'MDM_' + @parm_entity_type + '_TT';
			
			IF (lower(@parm_process_type) = lower('BULK')) 
			  BEGIN
			    SET @stgTbl = 'MDM_STAGING_'+CONVERT(VARCHAR,@parm_staging_id);
                SET @entityStTbl = 'MDM_BULK_ST_'+CONVERT(VARCHAR,@parm_staging_id);
			  END
			ELSE
			  BEGIN
			    SET @stgTbl = 'MDM_STAGING';
                SET @entityStTbl = 'MDM_' + @parm_entity_type + '_ST';
			  END
			
			SET @err_location = 51;
			SET @systimestamp = GETDATE();
			SET @errMessage = 'Start iteration for cluster ids from : '+CONVERT(VARCHAR,@firstClusterId)+' to '+CONVERT(VARCHAR,@lastClusterId)
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','',@errMessage,@systimestamp;
			
			IF (@parm_match_type = 0)			 
			  SET @mt0 = 1			 
			ELSE
			 BEGIN
			   CREATE TABLE #mt_tbl (id  INT IDENTITY(1,1),match_type  INT);
			   SET @strSQL = 'INSERT INTO #mt_tbl (match_type)
			               SELECT DISTINCT match_type FROM '+@entityStTbl+'
                            WHERE mdm_staging_id = @stgId
                              AND mdm_entity_cluster_id >= @firstCluster
                              AND mdm_entity_cluster_id <= @lastCluster';
               EXECUTE sp_executesql @strSQL,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId
			
			   SELECT @mtCount = max(id) FROM #mt_tbl;
			
			   SET @k = 1			
			
			   WHILE (@k <= @mtCount)
			    BEGIN
			      SELECT @mt = match_type FROM #mt_tbl WHERE id = @k
			      IF (@mt = 1) SET @mt1 = 1 
			      IF (@mt = 2) SET @mt2 = 1
			      IF (@mt = 3) SET @mt3 = 1
			      IF (@mt = 4) SET @mt4 = 1
			      SET @k = @k + 1
			    END
			    
			    DROP TABLE #mt_tbl
			 END			
			
			SET @systimestamp = GETDATE();
			SET @errMessage = 'Checked match types for cluster ids from : '+CONVERT(VARCHAR,@firstClusterId)+' to '+CONVERT(VARCHAR,@lastClusterId)
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','',@errMessage,@systimestamp;
			
	        /**************************************************************************************************************************************************************************
			 Update collapse clusters in transpose table
			**************************************************************************************************************************************************************************/
			
			SET @err_location = 60;
						
			SET @strSQL = 'UPDATE ' + @entityTtTbl +' 
                              SET mdm_entity_cluster_id = (SELECT DISTINCT st.mdm_entity_cluster_id 
                             FROM '+@stgTbl+' st 
                            WHERE st.mdm_staging_id = @stgId 
                              AND st.match_type = 2 
                              AND st.old_cluster_id <> st.mdm_entity_cluster_id 
                              AND st.mdm_entity_cluster_id >= @firstCluster
                              AND st.mdm_entity_cluster_id <= @lastCluster
                              AND ' + @entityTtTbl +'.mdm_entity_cluster_id = st.old_cluster_id) 
                         WHERE EXISTS (SELECT 1 
                                         FROM '+@stgTbl+' st 
                                        WHERE st.mdm_staging_id = @stgId 
                                          AND st.match_type = 2 
                                          AND st.old_cluster_id <> st.mdm_entity_cluster_id 
                                          AND st.mdm_entity_cluster_id >= @firstCluster
                                          AND st.mdm_entity_cluster_id <= @lastCluster
                                          AND ' + @entityTtTbl +'.mdm_entity_cluster_id = st.old_cluster_id) 
                          AND mdm_survivor = ''N'' 
                          AND (mdm_valid_to_dttm > getdate() OR mdm_valid_to_dttm IS NULL)';
               
            IF (@mt2 = 1)
                EXECUTE sp_executesql @strSQL,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId
	    
	        SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Update collapse clusters',@systimestamp;
	    
	        /**************************************************************************************************************************************************************************
		     Update move records in transpose table
		    **************************************************************************************************************************************************************************/
            SET @err_location = 70;
						
			SET @strSql = 'UPDATE '+@entityTtTbl+'
							  SET mdm_entity_cluster_id = (SELECT distinct st.mdm_entity_cluster_id 
																FROM '+@stgTbl+' st
																WHERE st.mdm_staging_id = @stgId
																	AND st.match_type > 2 
																	AND st.old_cluster_id <> st.mdm_entity_cluster_id 
																	AND st.mdm_entity_cluster_id >= @firstCluster
																	AND st.mdm_entity_cluster_id <= @lastCluster
																	AND '+@entityTtTbl+'.mdm_entity_id = st.mdm_entity_id)
							WHERE EXISTS (SELECT 1
											FROM '+@stgTbl+' st
											WHERE st.mdm_staging_id = @stgId
												AND st.match_type > 2 
												AND st.old_cluster_id <> st.mdm_entity_cluster_id 
												AND st.mdm_entity_cluster_id >= @firstCluster
												AND st.mdm_entity_cluster_id <= @lastCluster
												AND '+@entityTtTbl+'.mdm_entity_id = st.mdm_entity_id)'					
			IF (@mt3 = 1 OR @mt4 = 1)
			   EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId	
            
            SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Update move records',@systimestamp;
            
            /**************************************************************************************************************************************************************************
			 Retire old contributors and survivors from audit log
			**************************************************************************************************************************************************************************/
			-- Retire previous contributors: Note that if an update includes multiple records from the 
			-- same source system, they'll still appear in the cluster (but will be retired on the 
			-- next update that references the same ssid/ssrid)
	    
			SET @strOldSurvivorSQL = 'SELECT tt.mdm_entity_id
			              FROM '+@entityTtTbl+' tt, '+@entityStTbl+' st
						 WHERE st.mdm_staging_id = @stgId
						   AND st.mdm_entity_cluster_id >= @firstCluster
						   AND st.mdm_entity_cluster_id <= @lastCluster
						   AND st.mdm_survivor = ''N''
						   AND st.match_type > 0
						   AND tt.mdm_src_sys_id = st.mdm_src_sys_id
						   AND tt.mdm_src_sys_rec_id = st.mdm_src_sys_rec_id
                           AND tt.mdm_valid_to_dttm IS NULL
			           UNION
					   SELECT tt.mdm_entity_id
				         FROM '+@entityTtTbl+' tt
				        WHERE tt.mdm_entity_cluster_id IN (SELECT mdm_entity_cluster_id
														     FROM '+@entityStTbl+'
															WHERE mdm_staging_id = @stgId
															  AND match_type > 0
															  AND mdm_entity_cluster_id >= @firstCluster
															  AND mdm_entity_cluster_id <= @lastCluster
														   UNION
														   SELECT old_cluster_id
														     FROM '+@stgTbl+'
															WHERE mdm_staging_id = @stgId
															  AND mdm_entity_cluster_id >= @firstCluster
															  AND mdm_entity_cluster_id <= @lastCluster
															  AND match_type > 1
															  AND old_cluster_id <> mdm_entity_cluster_id)
				        AND tt.mdm_survivor = ''Y''
				        AND (tt.mdm_valid_to_dttm IS NULL OR tt.mdm_valid_to_dttm > getdate())';
								
			SET @err_location = 80;
			
			SET @strSQL = 'INSERT INTO mdm_'+@parm_entity_type+'_al (TABLE_NAME,RECORD_ID,USER_ACTION,MDM_MODIFIED_BY,MODIFIED_DTTM)
                           SELECT ''MDM_'+@parm_entity_type+'_TT'',contr.mdm_entity_id,''RETIRE'','''+@parm_userid+''',getdate()
							 FROM (' + @strOldSurvivorSQL + ') contr ';							
				
			IF (@mt1 = 1 OR @mt2 = 1 OR @mt3 = 1 OR @mt4 = 1)
			    EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int,@parm_userid nvarchar(513)',@parm_staging_id,@firstClusterId,@lastClusterId,@parm_userid; 
		 
		    SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Retire old contributors and survivors from audit log',@systimestamp;
		 
		    /************************************************************************************************************************************************************************** 
			 Retire old contributors and survivors from transpose table
			**************************************************************************************************************************************************************************/
			
			SET @err_location = 90;
			
			SET @strSql = 'UPDATE '+@entityTtTbl+' 
								SET mdm_valid_to_dttm = getdate() 
							WHERE mdm_entity_id IN (' + @strOldSurvivorSql + ')';
			
			IF (@mt1 = 1 OR @mt2 = 1 OR @mt3 = 1 OR @mt4 = 1)
			    EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId;

            SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Retire old contributors and survivors from transpose table',@systimestamp;

		    /**************************************************************************************************************************************************************************
			 Insert new records into transpose table
			**************************************************************************************************************************************************************************/
			
			SET @err_location = 100;	
			 
			-- insert into the transpose table
			IF (@mt0 = 1 AND lower(@parm_process_type) = lower('BULK'))
			 BEGIN
			   SET @strSql = 'INSERT INTO MDM_ENTITY_SEQ (mdm_staging_id,row_id) '+
                             'SELECT est.mdm_staging_id,est.row_id 
                                FROM '+@entityStTbl+' est, mdm_entity_cluster_seq cseq
                               WHERE cseq.mdm_staging_id = @stgId 
                                 AND cseq.seq >= @firstCluster 
								 AND cseq.seq <= @lastCluster
								 AND cseq.cid = est.cid';
               EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId
               
               SET @strSql = 'UPDATE STATISTICS MDM_ENTITY_SEQ'
               EXECUTE sp_executesql @strSql
               
			   SET @strSql = 'INSERT INTO '+@entityTtTbl+' (mdm_staging_id,mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,mdm_entity_cluster_id,mdm_survivor,mdm_valid_from_dttm' + @tt_columns + ') 
							SELECT est.mdm_staging_id,seq.seq,est.mdm_src_sys_id,
                               CASE WHEN est.mdm_src_sys_rec_id IS NULL THEN CONVERT(NVARCHAR,seq.seq) ELSE est.mdm_src_sys_rec_id END,
                               est.mdm_language_id,cseq.seq,est.mdm_survivor,getdate()' + @tt_columns + ' 
                              FROM '+@entityStTbl+' est, mdm_entity_seq seq, mdm_entity_cluster_seq cseq
					         WHERE cseq.mdm_staging_id = @stgId
				               AND cseq.seq >= @firstcluster
					           AND cseq.seq <= @lastCluster
					           AND est.cid = cseq.cid
					           AND est.mdm_staging_id = seq.mdm_staging_id
					           AND est.row_id = seq.row_id';
			 END
			ELSE
			 BEGIN
			   IF (@mt0 = 1) 
			     SET @mtClause = ' AND match_type = 0'
			   ELSE
			     SET @mtClause = ' AND match_type > 0'
			     
			   SET @strSql = 'INSERT INTO MDM_ENTITY_SEQ (mdm_staging_id,row_id) '+
                            'SELECT mdm_staging_id,row_id FROM '+@entityStTbl+' '+
                             'WHERE mdm_staging_id = @stgId 
                                AND mdm_entity_cluster_id >= @firstCluster 
								AND mdm_entity_cluster_id <= @lastCluster'+@mtClause;
               EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId
               
               IF (lower(@parm_process_type) = lower('BULK')) 
			    BEGIN
			      SET @strSql = 'UPDATE STATISTICS MDM_ENTITY_SEQ'
                  EXECUTE sp_executesql @strSql
			    END
			   
			   IF (@mt0 = 1) 
			     SET @mtClause = ' AND est.match_type = 0'
			   ELSE
			     SET @mtClause = ' AND est.match_type > 0'
			   
			   SET @strSql = 'INSERT INTO '+@entityTtTbl+' (mdm_staging_id,mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,mdm_entity_cluster_id,mdm_survivor,mdm_valid_from_dttm' + @tt_columns + ') 
							SELECT est.mdm_staging_id,seq.seq,est.mdm_src_sys_id,
                               CASE WHEN est.mdm_src_sys_rec_id IS NULL THEN CONVERT(NVARCHAR,seq.seq) ELSE est.mdm_src_sys_rec_id END,
                               est.mdm_language_id,est.mdm_entity_cluster_id,est.mdm_survivor,getdate()' + @tt_columns + ' 
                              FROM '+@entityStTbl+' est, mdm_entity_seq seq
					         WHERE est.mdm_staging_id = @stgId
				               AND est.mdm_entity_cluster_id >= @firstcluster
					           AND est.mdm_entity_cluster_id <= @lastCluster
					           AND est.mdm_staging_id = seq.mdm_staging_id
					           AND est.row_id = seq.row_id'+@mtClause;			   
			 END
			
			EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId 
			
		    SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Insert new records into transpose table',@systimestamp;
		 
		    /**************************************************************************************************************************************************************************  
			 Rename / Insert clusters in entity clusters table
			**************************************************************************************************************************************************************************/
			
			SET @err_location = 120;
			-- insert new clusters into the cluster table
			IF (@mt0 = 1 AND lower(@parm_process_type) = lower('BULK'))
			 BEGIN
			   SET @strSql = 'INSERT INTO mdm_entity_clusters
						   SELECT cseq.seq,est.mdm_entity_cluster_name, ''N''
						     FROM '+@entityStTbl+' est, mdm_entity_cluster_seq cseq
                            WHERE cseq.mdm_staging_id = @stgId
					          AND cseq.seq >= @firstCluster
                              AND cseq.seq <= @lastCluster
                              AND cseq.cid = est.cid
	                          AND mdm_survivor = ''Y''';
			 END
		    ELSE
		     BEGIN
			   SET @strSql = 'INSERT INTO mdm_entity_clusters
						   SELECT mdm_entity_cluster_id,mdm_entity_cluster_name, ''N''
						     FROM '+@entityStTbl+'
                            WHERE mdm_staging_id = @stgId
					          AND mdm_entity_cluster_id >= @firstCluster
                              AND mdm_entity_cluster_id <= @lastCluster
	                          AND mdm_survivor = ''Y''
                              AND (match_type = 0 OR match_type = 4)';
             END
            
            IF (@mt0 = 1 OR @mt4 = 1)                 
			    EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId 			            
			
			-- update existing clusters
			SET @strSql = 'MERGE INTO mdm_entity_clusters ec
		                   USING (SELECT mdm_entity_cluster_id, mdm_entity_cluster_name, match_type
				                    FROM '+@entityStTbl+'
				                   WHERE mdm_staging_id = @stgId
					                 AND mdm_entity_cluster_id >= @firstCluster
					                 AND mdm_entity_cluster_id <= @lastCluster
					                 AND mdm_survivor = ''Y''
					                 AND match_type > 0  AND match_type < 4) s
			                 ON (ec.mdm_entity_cluster_id = s.mdm_entity_cluster_id)
		                  WHEN MATCHED AND ec.mdm_entity_cluster_name <> s.mdm_entity_cluster_name THEN
			               UPDATE
				              SET ec.mdm_entity_cluster_name = s.mdm_entity_cluster_name;';
			           
			IF (@mt1 = 1 OR @mt2 = 1 OR @mt3 = 1)  
			    EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId 			            
			
			SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Rename / Insert clusters',@systimestamp;
						   
			/**************************************************************************************************************************************************************************
			 Insert collapse clusters into cluster mapping
			**************************************************************************************************************************************************************************/
			
			SET @err_location = 130;
			
			-- Insert into cluster mapping for match type 2 clusters
			IF (@mt2 = 1)
			  BEGIN
			    SET @strSql = 'INSERT INTO mdm_cluster_mapping
			                   SELECT DISTINCT old_cluster_id, mdm_entity_cluster_id
					             FROM '+@stgTbl+'
				                WHERE mdm_staging_id = @stgId 
				                  AND mdm_entity_cluster_id >= @firstCluster
				                  AND mdm_entity_cluster_id <= @lastCluster
				                  AND match_type = 2 
				                  AND old_cluster_id <> mdm_entity_cluster_id'
				EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId 			            
			  END
			SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Insert into cluster mapping',@systimestamp;
					
			/*************************************************************************************************************************************************************************
			 Update / Insert cluster member history
			**************************************************************************************************************************************************************************/
			
			SET @err_location = 140;			
			
	        IF (@mt2 = 1 OR @mt3 = 1 OR @mt4 = 1)
	          BEGIN
	            IF (@mt2 = 1)
	              BEGIN
	                SET @history_reason = 'C'
	                IF (@parm_comment IS NULL) SET @history_comment = 'Cluster Collapse'
	                   ELSE SET @history_comment = @parm_comment
	              END
	            ELSE 
	              BEGIN
	                SET @history_reason = 'M'
	                IF (@parm_comment IS NULL) SET @history_comment = 'Move Record'
	                   ELSE SET @history_comment = @parm_comment
	              END
	            
	            -- Retire any previous cluster member history records for match type 2 records
	            SET @strSql = 'UPDATE mdm_cluster_member_history
			                      SET mdm_valid_to_dttm = @curr_dttm
		                        WHERE mdm_entity_id IN (SELECT st.mdm_entity_id
											             FROM '+@stgTbl+' st
											            WHERE st.mdm_staging_id = @stgId
												          AND st.mdm_entity_cluster_id >= @firstCluster
												          AND st.mdm_entity_cluster_id <= @lastCluster
												          AND st.match_type > 1
												          AND st.old_cluster_id <> st.mdm_entity_cluster_id)
					              AND (mdm_valid_to_dttm > @curr_dttm OR mdm_valid_to_dttm IS NULL)';
			    EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int,@curr_dttm datetime',@parm_staging_id,@firstClusterId,@lastClusterId,@parm_current_dttm		            
	
	            -- Insert new cluster member history records
	            SET @strSql = 'INSERT INTO mdm_cluster_member_history (mdm_entity_cluster_id,mdm_entity_id,mdm_comment,mdm_reason,mdm_modified_dttm)
					         SELECT old_cluster_id,mdm_entity_id,@comment,@reason,@curr_dttm
					           FROM '+@stgTbl+'
					          WHERE mdm_staging_id = @stgId
						        AND mdm_entity_cluster_id >= @firstCluster
						        AND mdm_entity_cluster_id <= @lastCluster
						        AND match_type > 1
						        AND old_cluster_id <> mdm_entity_cluster_id';
			    EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int,@curr_dttm datetime,@comment nvarchar(200),@reason char(1)',@parm_staging_id,@firstClusterId,@lastClusterId,@parm_current_dttm,@history_comment,@history_reason	            
            END
            SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Update / Insert cluster member history',@systimestamp;

			/**************************************************************************************************************************************************************************
			 Remove retired clusters from forced clusters
			**************************************************************************************************************************************************************************/
			
			SET @err_location = 170;
			
			IF (@mt2 = 1)
			  BEGIN
			    -- remove retired clusters from forced clusters table
			    SET @strSql = 'UPDATE mdm_forced_clusters
				   SET original_cluster_id = (SELECT distinct -1 
											    FROM '+@stgTbl+'
											   WHERE mdm_staging_id = @stgId 
												 AND mdm_entity_cluster_id >= @firstCluster
												 AND mdm_entity_cluster_id <= @lastCluster
												 AND match_type = 2 
												 AND old_cluster_id <> mdm_entity_cluster_id
												 AND old_cluster_id = mdm_forced_clusters.original_cluster_id)
			     WHERE EXISTS (SELECT 1 
							     FROM '+@stgTbl+'
							    WHERE mdm_staging_id = @stgId 
								  AND mdm_entity_cluster_id >= @firstCluster
								  AND mdm_entity_cluster_id <= @lastCluster 
								  AND match_type = 2 
								  AND old_cluster_id <> mdm_entity_cluster_id
								  AND (old_cluster_id = mdm_forced_clusters.original_cluster_id))';
				EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId
								
			    SET @strSql = 'UPDATE mdm_forced_clusters
				   SET current_cluster_id = (SELECT distinct -1 
											   FROM '+@stgTbl+'
											  WHERE mdm_staging_id = @stgId 
												AND mdm_entity_cluster_id >= @firstCluster 
												AND mdm_entity_cluster_id <= @lastCluster 
												AND match_type = 2 
												AND old_cluster_id <> mdm_entity_cluster_id
												AND old_cluster_id = mdm_forced_clusters.current_cluster_id)
			     WHERE EXISTS (SELECT 1 
							     FROM '+@stgTbl+'
							    WHERE mdm_staging_id = @stgId
								  AND mdm_entity_cluster_id >= @firstCluster 
								  AND mdm_entity_cluster_id <= @lastCluster 
								  AND match_type = 2 
								  AND old_cluster_id <> mdm_entity_cluster_id
								  AND (old_cluster_id = mdm_forced_clusters.current_cluster_id))';
				EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId
			END
			
			SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Remove retired clusters from forced clusters',@systimestamp;
								
			/**************************************************************************************************************************************************************************
		     Remove retired clusters from relationships
		    **************************************************************************************************************************************************************************/

		    SET @err_location = 180;
			IF (@mt2 = 1 OR @mt4 = 1)
			  BEGIN
			    SET @strSql = 'UPDATE mdm_entity_rel
                   SET mdm_valid_to_dttm = getdate()
                 WHERE mdm_from_cluster_id IN (SELECT old_cluster_id
                                                 FROM '+@stgTbl+'
                                                WHERE mdm_staging_id = @stgId
                                                  AND mdm_entity_cluster_id >= @firstCluster
                                                  AND mdm_entity_cluster_id <= @lastCluster
                                                  AND (match_type = 2 OR match_type = 4)
                                                  AND old_cluster_id <> mdm_entity_cluster_id)
                    OR mdm_to_cluster_id IN (SELECT old_cluster_id
                                               FROM '+@stgTbl+'
                                              WHERE mdm_staging_id = @stgId
                                                AND mdm_entity_cluster_id >= @firstCluster
                                                AND mdm_entity_cluster_id <= @lastCluster
                                                AND (match_type = 2 OR match_type = 4)
                                                AND old_cluster_id <> mdm_entity_cluster_id)';
                EXECUTE sp_executesql @strSql,N'@stgId int,@firstCluster int,@lastCluster int',@parm_staging_id,@firstClusterId,@lastClusterId
			END
			
			SET @systimestamp = GETDATE();
			--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','','Remove retired clusters from relationships',@systimestamp;
														
			/**************************************************************************************************************************************************************************
			 Commit
			**************************************************************************************************************************************************************************/
			
			SET @err_location = 190;
		    COMMIT TRAN goLineTran
		
		SET @systimestamp = GETDATE();
	    SET @errMessage = 'Iteration commited for cluster ids from : '+CONVERT(VARCHAR,@firstClusterId)+' to '+CONVERT(VARCHAR,@lastClusterId)
		--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','',@errMessage,@systimestamp;
		                      
    END TRY
    BEGIN CATCH
        ROLLBACK TRAN;
		DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)
		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = 'Location: '+CONVERT(VARCHAR,@err_location)+' : '+@errorProc+ ' : '+@error_messages+ 
									     ' : '+@errorSeverityString+
									     ' : '+@mdm_error_status+
									     ' : '+@errorStateString
		     -- Log Error
		     --NOTE Do we log old or new id?
		EXEC [mdm_log_error] 'MDM_ENTITY_GOLIVE',null,@err_location,'mdm_staging_id','mdm_entity_golive',@mdm_error_status,@mdm_error_messages,@errorDate

		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
    END CATCH         
  END 
  
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_deleteProcessedRecords]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_deleteProcessedRecords]
GO

CREATE PROCEDURE [mdm_deleteProcessedRecords] (@parm_entity_type       VARCHAR(23),
								               @parm_staging_id        INT,
								               @parm_startCluster      INT,
								               @parm_endCluster        INT,
								               @parm_process_type      VARCHAR(30),
								               @parm_increment         INT) AS
 DECLARE @err_location  INT;
 DECLARE @strSQL        NVARCHAR(2000);
 DECLARE @firstIdx      INT;
 DECLARE @lastIdx       INT;
 DECLARE @stgTbl        VARCHAR(30);
 DECLARE @entityStgTbl  VARCHAR(30);
BEGIN
  SET NOCOUNT ON  
  BEGIN TRY 
    IF (lower(@parm_entity_type) = lower('BULK'))
      BEGIN
        SET @stgTbl = 'MDM_STAGING_'+CONVERT(VARCHAR,@parm_staging_id);
        SET @entityStgTbl = 'MDM_BULK_ST_'+CONVERT(VARCHAR,@parm_staging_id); 
      END
    ELSE
      BEGIN
        SET @stgTbl = 'MDM_STAGING';
        SET @entityStgTbl = 'MDM_'+@parm_entity_type+'_ST';
      END
    
    IF (@parm_endCluster IS NOT NULL) 
      BEGIN
        SET @firstIdx = @parm_startCluster
        SET @lastIdx = @firstIdx + @parm_increment
        IF (@lastIdx > @parm_endCluster)
          BEGIN
            SET @lastIdx = @parm_endCluster
          END
      
        WHILE (@firstIdx <= @parm_endCluster)
          BEGIN
            SET @err_location = 10;
            SET @strSQL = 'DELETE FROM '+@stgTbl+' WHERE mdm_staging_id = @staging_id
                              AND mdm_entity_cluster_id >= @firstIdx AND mdm_entity_cluster_id <= @lastIdx';
            EXECUTE sp_executesql @strSql,N'@staging_id int,@firstIdx int,@lastIdx int',@parm_staging_id,@firstIdx,@lastIdx;        
         
            SET @err_location = 20;
            SET @strSql = 'DELETE FROM '+@entityStgTbl+' '+
                          ' WHERE mdm_staging_id = @staging_id '+
                             'AND mdm_entity_cluster_id >= @firstIdx AND mdm_entity_cluster_id <= @lastIdx';
            EXECUTE sp_executesql @strSql,N'@staging_id int,@firstIdx int,@lastIdx int',@parm_staging_id,@firstIdx,@lastIdx;        
    
            SET @firstIdx = @lastIdx + 1
            SET @lastIdx = @firstIdx + @parm_increment
            IF (@lastIdx > @parm_endCluster)
              BEGIN
                SET @lastIdx = @parm_endCluster
              END
          END
      END
  END TRY
  BEGIN CATCH 
		DECLARE @error_messages       VARCHAR(2000);
        DECLARE @errorSeverity        INT;
        DECLARE @errorState           INT;
        DECLARE @errorStatus          INT;
        DECLARE @errorProc            VARCHAR(150);
        DECLARE @errorDate            DATETIME;
        DECLARE @mdm_error_status     VARCHAR(10);
        DECLARE @mdm_error_messages   VARCHAR(4000);
        DECLARE @errorSeverityString  VARCHAR(100);
        DECLARE @errorStateString     VARCHAR(100);
		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()		

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = 'Location: '+CONVERT(VARCHAR,@err_location)+' : '+@errorProc+ ' : '+@error_messages+ 
									     ' : '+@errorSeverityString+
									     ' : '+@mdm_error_status+
									     ' : '+@errorStateString
		
		     -- Log Error
		     --NOTE Do we log old or new id?
		EXEC [mdm_log_error] parm_entity_type,'mdm_staging_id',@parm_staging_id,null,'mdm_deleteProcessedRecords',@mdm_error_status,@mdm_error_messages,@errorDate
       
		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
    END CATCH        
  END 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_staging_golive]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_staging_golive]
GO

CREATE PROCEDURE [mdm_staging_golive] (@parm_staging_id       INT,
								      @parm_entity_type_id   INT,
								      @increment             INT,
								      @parm_userid NVARCHAR(513),
								      @parm_process_type VARCHAR(30),
								      @parm_comment NVARCHAR(200) = NULL) AS
								    
  DECLARE @tt_columns           NVARCHAR(MAX);
  DECLARE @parm_current_dttm    DATETIME;
  DECLARE @parm_entity_type     VARCHAR(23);
  DECLARE @attr_name            VARCHAR(50);	
  DECLARE @strSql               NVARCHAR(MAX);  
  DECLARE @stgTbl               VARCHAR(30);
  DECLARE @entityStgTbl         VARCHAR(30);
  DECLARE @entityCcTbl          VARCHAR(30);
  DECLARE @firstIdx             INT;
  DECLARE @lastIdx              INT;
  DECLARE @firstCluster         INT;
  DECLARE @lastCluster          INT;
  DECLARE @cluster_count_0      INT;
  DECLARE @cluster_count_1      INT;
  DECLARE @startCluster         INT;
  DECLARE @endCluster           INT;
  DECLARE @process_id           INT;	
  DECLARE @get_attrs            CURSOR;
  DECLARE @timestamp            DATETIME;
  DECLARE @errtimestamp         DATETIME;
  DECLARE @err_location         INT;
  DECLARE @errMessage           VARCHAR(100);
  DECLARE @num_rows     INT;
    
BEGIN
  SET NOCOUNT ON  
  BEGIN TRY   
    SELECT @timestamp = GETDATE()   
    BEGIN TRAN
      SET @err_location = 10;
	  SET @errtimestamp = GETDATE();
	  SET @errMessage = 'Starting staging id : '+CONVERT(VARCHAR,@parm_staging_id)
	  --EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','',@errMessage,@errtimestamp;
      
      INSERT INTO [MDM_PROCESS_CONTROL]
			   (
			    mdm_staging_id
			   ,process_name
			   ,mdm_entity_type_id
			   ,process_running
			   ,process_user
			   ,process_start_dttm	)		   
		 VALUES
			   (
				@parm_staging_id
			   ,'MAKE DATA LIVE'
			   ,@parm_entity_type_id
			   ,'Y'
			   ,@parm_userid
			   ,@timestamp)
    COMMIT TRAN
    
    SELECT @process_id = process_id 
		 FROM mdm_process_control
		WHERE process_name = 'MAKE DATA LIVE'
		  AND mdm_entity_type_id = @parm_entity_type_id
		  AND mdm_staging_id = @parm_staging_id
		  AND process_running = 'Y'
		  AND process_start_dttm = @timestamp
      
    SET @err_location = 20;
    -- Get entity type and timestamp information
    SELECT @parm_current_dttm = GETDATE();
    SELECT @parm_entity_type = mdm_entity_type 
      FROM mdm_entity_types 
     WHERE mdm_entity_type_id=@parm_entity_type_id 
       AND mdm_valid_from_dttm < @parm_current_dttm 
       AND (mdm_valid_to_dttm is null or mdm_valid_to_dttm > @parm_current_dttm);
    
    -- Construct column string
    SET @get_attrs=CURSOR FAST_FORWARD FOR WITH tree (mdm_entity_type_id, mdm_parent_id) AS 
        (SELECT et.mdm_entity_type_id, et.mdm_parent_id
        FROM mdm_entity_types et 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
          AND et.MDM_ENTITY_TYPE_ID = @parm_entity_type_id 
        UNION ALL 
        SELECT et.mdm_entity_type_id, et.mdm_parent_id 
        FROM mdm_entity_types et, tree t 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
          AND et.MDM_ENTITY_TYPE_ID = t.mdm_parent_id)    
    SELECT mdm_attr_name FROM mdm_attrs
       WHERE mdm_entity_type_id IN (SELECT mdm_entity_type_id FROM tree)
         AND mdm_valid_from_dttm < @parm_current_dttm and (mdm_valid_to_dttm is null or mdm_valid_to_dttm > @parm_current_dttm)
         AND mdm_attr_id > 0;
       
    SET @tt_columns = '';
    OPEN @get_attrs;
    FETCH NEXT FROM @get_attrs INTO @attr_name;
    WHILE  @@FETCH_STATUS = 0 BEGIN        
        SET @tt_columns = @tt_columns+','+@attr_name;  
    FETCH NEXT FROM @get_attrs INTO @attr_name;
    END   
    CLOSE @get_attrs;
    DEALLOCATE @get_attrs;    
    
    -- Assign new cluster ids
    SET @err_location = 30;
    IF (lower(@parm_process_type) = lower('BULK')) 
      BEGIN
		SET @stgTbl = 'MDM_STAGING_'+CONVERT(VARCHAR,@parm_staging_id);
        SET @entityStgTbl = 'MDM_BULK_ST_'+CONVERT(VARCHAR,@parm_staging_id);
        SET @entityCcTbl = 'MDM_BULK_CC_'+CONVERT(VARCHAR,@parm_staging_id);
        
        --Clear sequence data from any failed runs with same staging id
        DELETE FROM mdm_entity_cluster_seq WHERE mdm_staging_id = @parm_staging_id
        
		SET @strSql = 'INSERT INTO mdm_entity_cluster_seq (mdm_staging_id,cid) 
			 SELECT DISTINCT mdm_staging_id,cid FROM '+@entityStgTbl+'
			  WHERE mdm_staging_id = @stgId
				AND match_type = 0';
		EXECUTE sp_executesql @strSQL,N'@stgId int',@parm_staging_id
		
		SET @strSql = 'UPDATE STATISTICS MDM_ENTITY_CLUSTER_SEQ'
        EXECUTE sp_executesql @strSql
        
        EXEC [mdm_getStats] @parm_entity_type,'POST_CLUSTER',@parm_staging_id	
      END
    ELSE
      BEGIN
        SET @stgTbl = 'MDM_STAGING';
        SET @entityStgTbl = 'MDM_'+@parm_entity_type+'_ST';
      END
    SET @errtimestamp = GETDATE();
	SET @errMessage = 'Assigned new cluster Ids : '+CONVERT(VARCHAR,@parm_staging_id)
	--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','',@errMessage,@errtimestamp;
   
    -- Load the clusters for the staging set
    SET @err_location = 40;
	    
    CREATE TABLE #tbl_cluster_0 (cluster_num  INT IDENTITY(1,1),cluster_id  INT);
    CREATE TABLE #tbl_cluster_1 (cluster_num  INT IDENTITY(1,1),cluster_id  INT);
    
    IF (lower(@parm_process_type) = lower('BULK')) 
      BEGIN
        INSERT INTO #tbl_cluster_0 (cluster_id) 
          SELECT seq FROM mdm_entity_cluster_seq
           WHERE mdm_staging_id = @parm_staging_id order by seq;    
      END
    ELSE
      BEGIN
        SET @strSql = 'INSERT INTO #tbl_cluster_0 (cluster_id) 
                     SELECT DISTINCT mdm_entity_cluster_id FROM '+@entityStgTbl+'
                      WHERE mdm_staging_id = @stgId AND match_type = 0 order by mdm_entity_cluster_id';
        EXECUTE sp_executesql @strSql,N'@stgId int',@parm_staging_id 
      END
    
    SET @strSql = 'INSERT INTO #tbl_cluster_1 (cluster_id) 
                     SELECT DISTINCT mdm_entity_cluster_id FROM '+@entityStgTbl+'
                      WHERE mdm_staging_id = @stgId AND match_type > 0 order by mdm_entity_cluster_id';
    EXECUTE sp_executesql @strSql,N'@stgId int',@parm_staging_id    
    
    SET @errtimestamp = GETDATE();
	SET @errMessage = 'Loaded cluster ids for processing : '+CONVERT(VARCHAR,@parm_staging_id)
	--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','',@errMessage,@errtimestamp;
    
    
    SET @err_location = 50;
    -- Iterate over existing clusters
    SET @firstIdx = 1
    SET @lastIdx = @firstIdx + @increment
    SELECT @cluster_count_1 = COUNT(1) FROM #tbl_cluster_1
    
    IF (@lastIdx > @cluster_count_1)
      BEGIN
         SET @lastIdx = @cluster_count_1
      END
    
    WHILE (@firstIdx <= @cluster_count_1)
      BEGIN
         SELECT @firstCluster = cluster_id FROM #tbl_cluster_1 WHERE cluster_num = @firstIdx
         SELECT @lastCluster = cluster_id FROM #tbl_cluster_1 WHERE cluster_num = @lastIdx         
         IF (@firstIdx = 1)
           SET @startCluster = @firstCluster
         EXEC [mdm_entity_golive] @firstCluster,@lastCluster,@parm_staging_id,@parm_entity_type_id,@tt_columns,@parm_entity_type,@parm_current_dttm,@parm_userid,@parm_process_type,1,@parm_comment
         SET @endCluster = @lastCluster
         IF (@firstIdx = 1) AND (lower(@parm_process_type) = lower('BULK'))
           EXEC [mdm_getStats] @parm_entity_type,'POST_LIVE'
         
         SET @firstIdx = @lastIdx + 1
         SET @lastIdx = @firstIdx + @increment
         IF (@lastIdx > @cluster_count_1)
           BEGIN
             SET @lastIdx = @cluster_count_1
           END         
      END
    DROP TABLE #tbl_cluster_1;   
    
    SET @err_location = 52;
    -- Iterate over new clusters
    SET @firstIdx = 1
    SET @lastIdx = @firstIdx + @increment
    SELECT @cluster_count_0 = COUNT(1) FROM #tbl_cluster_0
    
    IF (@lastIdx > @cluster_count_0)
      BEGIN
         SET @lastIdx = @cluster_count_0
      END
    
    WHILE (@firstIdx <= @cluster_count_0)
      BEGIN
         SELECT @firstCluster = cluster_id FROM #tbl_cluster_0 WHERE cluster_num = @firstIdx
         SELECT @lastCluster = cluster_id FROM #tbl_cluster_0 WHERE cluster_num = @lastIdx         
         IF (@firstIdx = 1) AND (@startCluster IS NULL)
           SET @startCluster = @firstCluster
         SET @err_location = 53;
         EXEC [mdm_entity_golive] @firstCluster,@lastCluster,@parm_staging_id,@parm_entity_type_id,@tt_columns,@parm_entity_type,@parm_current_dttm,@parm_userid,@parm_process_type,0,@parm_comment
         SET @err_location = 54;
         SET @endCluster = @lastCluster
         IF (@firstIdx = 1) AND (lower(@parm_process_type) = lower('BULK'))
           EXEC [mdm_getStats] @parm_entity_type,'POST_LIVE'
         
         SET @firstIdx = @lastIdx + 1
         SET @lastIdx = @firstIdx + @increment
         IF (@lastIdx > @cluster_count_0)
           BEGIN
             SET @lastIdx = @cluster_count_0
           END         
      END
    
    DROP TABLE #tbl_cluster_0;   
    
    -- Iterate to delete from entity staging table       
    SET @err_location = 200;
	IF (lower(@parm_process_type) = lower('BULK'))
	  BEGIN
	    -- Delete staging tables
        SET @strSQL = 'DROP TABLE '+@entityStgTbl;
        EXECUTE sp_executesql @strSQL;
        
        SET @strSQL = 'DROP TABLE '+@entityCcTbl;
        EXECUTE sp_executesql @strSQL;

        SET @strSQL = 'DROP TABLE '+@stgTbl;
        EXECUTE sp_executesql @strSQL;  
        
        DELETE FROM mdm_entity_seq WHERE mdm_staging_id = @parm_staging_id;
        
        DELETE FROM mdm_entity_cluster_seq WHERE mdm_staging_id = @parm_staging_id
	  END
	ELSE
	  EXEC [mdm_deleteProcessedRecords] @parm_entity_type,@parm_staging_id,@startCluster,@endCluster,'MRM',@increment
      
    SET @errtimestamp = GETDATE();
	SET @errMessage = 'Cleared staging tables : '+CONVERT(VARCHAR,@parm_staging_id)
	--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','',@errMessage,@errtimestamp;
      
    EXEC [mdm_end_process_control] @process_id,'Y'
    
    SET @errtimestamp = GETDATE();
	SET @errMessage = 'Finished : '+CONVERT(VARCHAR,@parm_staging_id)
	--EXECUTE mdm_log_error 'MDM_ENTITY_GOLIVE',null,0,'mdm_entity_id','mdm_entity_goLive','',@errMessage,@errtimestamp;
    
    SELECT 'Y' AS process_complete
    
  END TRY
  BEGIN CATCH 
		DECLARE @error_messages       VARCHAR(2000);
        DECLARE @errorSeverity        INT;
        DECLARE @errorState           INT;
        DECLARE @errorStatus          INT;
        DECLARE @errorProc            VARCHAR(150);
        DECLARE @errorDate            DATETIME;
        DECLARE @mdm_error_status     VARCHAR(10);
        DECLARE @mdm_error_messages   VARCHAR(4000);
        DECLARE @errorSeverityString  VARCHAR(100);
        DECLARE @errorStateString     VARCHAR(100);
		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()		

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = 'Location: '+CONVERT(VARCHAR,@err_location)+' : '+@errorProc+ ' : '+@error_messages+ 
									     ' : '+@errorSeverityString+
									     ' : '+@mdm_error_status+
									     ' : '+@errorStateString
		
		     -- Log Error
		     --NOTE Do we log old or new id?
		EXEC [mdm_log_error] tbl_name,null,@parm_staging_id,'mdm_staging_id','mdm_staging_goLive',@mdm_error_status,@mdm_error_messages,@errorDate

        EXEC [mdm_deleteProcessedRecords] @parm_entity_type,@parm_staging_id,@startCluster,@endCluster,@parm_process_type,@increment   
          
        EXEC [mdm_end_process_control] @process_id,'N'
		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
    END CATCH        
  END 
 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_buildSearchVw]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_buildSearchVw]
GO

CREATE PROCEDURE [mdm_buildSearchVw](@parm_entity_type_id    INT,
									  @parm_entity_type    VARCHAR(25),
									  @parm_is_abstract   CHAR(1)) AS
DECLARE @firstSubclass  INT;
DECLARE @strSQL         NVARCHAR(MAX);
DECLARE @subclassViewName  VARCHAR(50);
DECLARE @isAbstractSubclass CHAR(1);
DECLARE @sv_name   VARCHAR(50);
DECLARE @get_subclasses CURSOR;
DECLARE @attr_count INT;
DECLARE @attr_name  VARCHAR(50);
DECLARE @tt_columns NVARCHAR(MAX);
DECLARE @j INT;
DECLARE @actualEntityTypeId INT;
SET NOCOUNT ON  
BEGIN TRY
    
    SET @sv_name = 'MDM_'+ upper(@parm_entity_type) + '_SV';
    
    -- Build column string
    CREATE TABLE #colTbl (id int identity(1,1),colStrId int,colStrValue varchar(30));
    INSERT INTO #colTbl (colStrId,colStrValue)
       EXEC [mdm_getAttrs] @parm_entity_type,'D'
    SELECT @attr_count = COUNT(1) FROM #colTbl
    SET @tt_columns = ',MDM_SURVIVOR' 
    SET @j = 1
    WHILE (@j <= @attr_count) BEGIN
       SELECT @attr_name = colStrValue FROM #colTbl WHERE id = @j
       IF (LOWER(@attr_name) <> LOWER('MDM_ENTITY_TYPE_ID'))
         SET @tt_columns = @tt_columns+','+@attr_name
       SET @j = @j+1
    END
    DROP TABLE #colTbl
    
    SET @firstSubclass=0;
    
    -- Create initial view string
    SET @strSQL = 'create view '+ @sv_name + ' as ';
    IF (@parm_is_abstract = 'N')BEGIN
       SET @strSQL = @strSQL +' select max(isnull(mdm_valid_to_dttm,cast(''12/31/9999'' as datetime))) over (partition by mdm_entity_cluster_id) max_valid_to_dttm,'+CONVERT(VARCHAR,@parm_entity_type_id)+' as mdm_entity_type_id'+@tt_columns
                   +' from MDM_' + @parm_entity_type + '_TT ';
    END;
    
    -- Append subclass union strings
    SET @get_subclasses=CURSOR FAST_FORWARD FOR WITH tree (mdm_entity_type, mdm_entity_type_id, mdm_abstract, mdm_parent_id) AS 
        (SELECT et.mdm_entity_type, et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id
        FROM mdm_entity_types et 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
          AND et.MDM_ENTITY_TYPE_ID = @parm_entity_type_id 
        UNION ALL 
        SELECT et.mdm_entity_type, et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id 
        FROM mdm_entity_types et, tree t 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null)
          AND et.mdm_valid_from_dttm <= GETDATE()
          AND et.mdm_parent_id = t.mdm_entity_type_id
          AND et.mdm_entity_type_id > 0)
        SELECT 'MDM_'+upper(mdm_entity_type)+'_TT', mdm_abstract,mdm_entity_type_id FROM tree WHERE mdm_entity_type_id <> @parm_entity_type_id;
        
     OPEN @get_subclasses;
      FETCH NEXT FROM @get_subclasses INTO @subclassViewName,@isAbstractSubclass,@actualEntityTypeId;
        WHILE @@FETCH_STATUS=0 BEGIN
          IF (@isAbstractSubclass='N') 
            BEGIN
              IF (@firstSubclass > 0) 
                BEGIN
                 SET @strSQL = @strSQL+' union ';
                END
              ELSE
                BEGIN
                 SET @firstSubclass=1;
                 IF (@parm_is_abstract = 'N') BEGIN
                   SET @strSQL = @strSQL+' union ';
                 END
                END;
              SET @strSQL = @strSQL +'select max(isnull(mdm_valid_to_dttm,cast(''12/31/9999'' as datetime))) over (partition by mdm_entity_cluster_id) max_valid_to_dttm,'+CONVERT(VARCHAR,@actualEntityTypeId)+' as mdm_entity_type_id'+@tt_columns
                   +' from '+@subclassViewName+' ';
          END; 
          FETCH NEXT FROM @get_subclasses INTO @subclassViewName,@isAbstractSubclass,@actualEntityTypeId;
        END;
     CLOSE @get_subclasses;
     DEALLOCATE @get_subclasses;
     
     IF (@firstSubclass > 0) OR (@parm_is_abstract = 'N')
      BEGIN 
         IF EXISTS (SELECT * FROM sys.objects WHERE name = @sv_name and type = 'V')
            EXEC ('DROP VIEW [' + @sv_name + ']')
            
         EXEC (@strSQL);
      END;       
    
END TRY
BEGIN CATCH     
       DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	   SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	   -- Log Error
	   EXEC [mdm_log_error] 'MDM_ENTITY_TYPES',null,@parm_entity_type_id,'parm_entity_type_id','mdm_buildSearchVw',@mdm_error_status,@mdm_error_messages,@errorDate
	   -- Raise Error
	   RAISERROR (@error_messages, @errorSeverity, @errorState)
	END CATCH  
	


GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_createStagingTables]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_createStagingTables]
GO

CREATE PROCEDURE [mdm_createStagingTables](@parm_entity_type_id  INT,
                                       @parm_entity_type  VARCHAR(23),
                                       @parm_staging_id  INT = NULL) AS
 DECLARE @err_location INT;
 DECLARE @get_attrs CURSOR; 
 DECLARE @entitySt_tbl VARCHAR(30);
 DECLARE @entityCc_tbl VARCHAR(30);
 DECLARE @stg_tbl      VARCHAR(30);
 DECLARE @num_rows     INT;
 DECLARE @get_clusterCols CURSOR;
 DECLARE @attr_name VARCHAR(50);
 DECLARE @attr_datatype VARCHAR(50);
 DECLARE @attr_nullable VARCHAR(10);
 DECLARE @attr_defn     VARCHAR(100);
 DECLARE @attr_encrypt CHAR(1);
 DECLARE @parent_id    INT;
 DECLARE @err_staging_id INT;
 SET NOCOUNT ON;
BEGIN TRY
 IF (@parm_staging_id IS NULL)
   BEGIN
     SET @entitySt_tbl = 'MDM_'+@parm_entity_type+'_ST'
     SET @entityCc_tbl = 'MDM_'+@parm_entity_type+'_CC'
     SET @stg_tbl = 'MDM_STAGING'
     SET @err_staging_id = 0
   END
 ELSE
   BEGIN
     SET @entitySt_tbl = 'MDM_BULK_ST_'+CONVERT(VARCHAR,@parm_staging_id)
     SET @entityCc_tbl = 'MDM_BULK_CC_'+CONVERT(VARCHAR,@parm_staging_id)
     SET @stg_tbl = 'MDM_STAGING_' +CONVERT(VARCHAR,@parm_staging_id)
     SET @err_staging_id = @parm_staging_id
   END
   
   -- If the staging table doesn't already exist, create it.
   SELECT @num_rows=COUNT(1) FROM sys.objects WHERE lower(name) = lower(@entitySt_tbl);
   IF (@num_rows = 0) 
     BEGIN
       SET @err_location = 10;
       EXEC ('CREATE TABLE [' + @entitySt_tbl + '] 
          ([MDM_STAGING_ID] INT NOT NULL,
          [ROW_ID] INT NOT NULL,
          [MDM_ENTITY_CLUSTER_ID] INT NULL,
          [MDM_ENTITY_CLUSTER_NAME] NVARCHAR(100) NOT NULL,
          [MDM_SURVIVOR] CHAR(1) NOT NULL,
          [MATCH_TYPE] INT NOT NULL,
          [CID] INT NOT NULL,
          [ROW_PROCESSED] CHAR(1) DEFAULT ''N'' NOT NULL,
          [MDM_SRC_SYS_ID] INT NOT NULL,
          [MDM_SRC_SYS_REC_ID] NVARCHAR(200),
          [MDM_LANGUAGE_ID] INT NOT NULL )')     
     END
       
       SET @err_location = 20;
       -- Add columns
       SET @get_attrs=CURSOR FAST_FORWARD FOR WITH tree (mdm_entity_type_id, mdm_parent_id, mdm_label_attr_id) AS 
              (SELECT et.mdm_entity_type_id, et.mdm_parent_id, et.mdm_label_attr_id 
				 FROM mdm_entity_types et 
			    WHERE (et.mdm_valid_to_dttm > GETDATE() OR et.mdm_valid_to_dttm IS NULL) 
				  AND et.MDM_ENTITY_TYPE_ID = @parm_entity_type_id 
               UNION ALL 
			   SELECT et.mdm_entity_type_id, et.mdm_parent_id, et.mdm_label_attr_id 
                 FROM mdm_entity_types et, tree t 
                WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
                  AND et.MDM_ENTITY_TYPE_ID = t.mdm_parent_id)

              SELECT a.mdm_attr_name,a.mdm_encrypt_flag,a.mdm_parent_attr_id,
                     CASE 
                       WHEN (t.mdm_db_datatype='VARCHAR' or t.mdm_db_datatype = 'NVARCHAR') THEN t.mdm_db_datatype+'('+CONVERT(VARCHAR, CASE a.mdm_attr_length WHEN 0 THEN 1 ELSE a.mdm_attr_length END)+')'
                       ELSE t.mdm_db_datatype END AS datatype,
                     CASE 
                       WHEN a.mdm_required='Y'  
					   or a.mdm_attr_id IN (select mdm_label_attr_id from tree)
					   THEN ' NOT NULL CHECK (len('+a.mdm_attr_name+')>=1)'
                       ELSE '' END AS nullable
               FROM mdm_attrs a, mdm_attr_datatypes t 
              WHERE a.mdm_attr_id > 0 
                AND a.mdm_attr_datatype_id=t.mdm_attr_datatype_id and a.mdm_entity_type_id IN (select mdm_entity_type_id from tree)
                AND a.mdm_valid_from_dttm <= GETDATE() 
                AND (a.mdm_valid_to_dttm > GETDATE() OR a.mdm_valid_to_dttm IS NULL)
              ORDER BY a.mdm_attr_name;
              
       OPEN @get_attrs;
       FETCH NEXT FROM @get_attrs INTO @attr_name,@attr_encrypt,@parent_id,@attr_datatype,@attr_nullable;
       WHILE @@FETCH_STATUS=0 BEGIN        
          IF (@attr_encrypt='Y' and @parent_id is null) 
            BEGIN          
              SET @attr_datatype = 'VARBINARY(1600)'
            END
		      
          SET @attr_defn=upper(@attr_name)+' '+@attr_datatype+' '+@attr_nullable;
           
          SELECT @num_rows=count(1) FROM syscolumns 
            WHERE lower(name)=lower(@attr_name) and id=(SELECT id FROM sysobjects WHERE type='U' AND lower(name)=lower(@entitySt_tbl))
          IF (@num_rows=0) BEGIN
           EXEC('ALTER TABLE ['+@entitySt_tbl+'] ADD '+@attr_defn);
          END;
       FETCH NEXT FROM @get_attrs INTO @attr_name,@attr_encrypt,@parent_id,@attr_datatype,@attr_nullable;
       END;
       CLOSE @get_attrs;
       DEALLOCATE @get_attrs;
        
   -- If cluster condition table doesn't already exist, create it.
   SELECT @num_rows=COUNT(1) FROM sys.objects WHERE lower(name) = lower(@entityCc_tbl);
   IF (@num_rows = 0) BEGIN
     SET @err_location = 30;
     EXEC ('CREATE TABLE [' + @entityCc_tbl + '] 
          ([MDM_STAGING_ID] INT NOT NULL,
          [ROW_ID] INT NOT NULL)')
    END;
          
     -- Add cluster columns      
     SET @get_clusterCols=CURSOR FAST_FORWARD FOR WITH tree (mdm_entity_type_id, mdm_parent_id, mdm_label_attr_id) AS 
          (SELECT et.mdm_entity_type_id, et.mdm_parent_id, et.mdm_label_attr_id
			 FROM mdm_entity_types et 
			WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
			  AND et.MDM_ENTITY_TYPE_ID = @parm_entity_type_id
           UNION ALL 
           SELECT et.mdm_entity_type_id, et.mdm_parent_id, et.mdm_label_attr_id
			 FROM mdm_entity_types et, tree t 
            WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
			  AND et.MDM_ENTITY_TYPE_ID = t.mdm_parent_id)          
         SELECT DISTINCT vw.mdm_attr_name, vw.datatype, vw.nullable
           FROM (SELECT ccon.mdm_entity_type_id,
				  a.mdm_attr_name,
                  ccon.mdm_sort_order,
                  CASE
                   WHEN (ad.mdm_db_datatype='VARCHAR' OR ad.mdm_db_datatype='NVARCHAR') THEN ad.mdm_db_datatype+'('+CONVERT(VARCHAR, CASE a.mdm_attr_length WHEN 0 THEN 1 ELSE a.mdm_attr_length END)+')'
                   ELSE ad.mdm_db_datatype END AS datatype,
                  CASE
                   WHEN a.mdm_required='Y' 
				   or a.mdm_attr_id IN (select mdm_label_attr_id from tree)
				   THEN 'NOT NULL'
                   ELSE 'NULL'END AS nullable
              FROM mdm_cluster_conditions ccon,
                   mdm_cluster_criteria ccri,
                   mdm_attrs a,
                   mdm_attr_datatypes ad
             WHERE ccon.mdm_cluster_condition_id = ccri.mdm_cluster_condition_id
               AND ccri.mdm_attr_id = a.mdm_attr_id
               AND a.mdm_valid_from_dttm <= GETDATE()
               AND (a.mdm_valid_to_dttm > GETDATE() OR a.mdm_valid_to_dttm IS NULL)
               AND a.mdm_attr_datatype_id = ad.mdm_attr_datatype_id) vw
         WHERE vw.mdm_entity_type_id IN (SELECT mdm_entity_type_id FROM tree)
         ORDER BY vw.mdm_attr_name;
      
      SET @err_location = 40;
      OPEN @get_clusterCols
      FETCH NEXT FROM @get_clusterCols INTO @attr_name,@attr_datatype,@attr_nullable;
      WHILE @@FETCH_STATUS=0 BEGIN
        SELECT @num_rows=count(1) FROM syscolumns 
         WHERE lower(name)=lower(@attr_name) and id=(SELECT id FROM sysobjects WHERE type='U' and lower(name)=lower(@entityCc_tbl))
             
        SET @attr_defn=upper(@attr_name)+' '+@attr_datatype+' '+@attr_nullable;    
          
        IF (@num_rows=0) BEGIN            
           EXEC('ALTER TABLE ['+@entityCc_tbl+'] ADD '+@attr_defn);              
        END;     
      FETCH NEXT FROM @get_clusterCols INTO @attr_name,@attr_datatype,@attr_nullable;
      END
      CLOSE @get_clusterCols;
      DEALLOCATE @get_clusterCols;        
      
   -- If staging table doesn't already exist, create it.
   SELECT @num_rows=COUNT(1) FROM sys.objects WHERE lower(name) = lower(@stg_tbl);
   IF (@num_rows = 0) BEGIN
     SET @err_location = 50;
     EXEC ('CREATE TABLE [' + @stg_tbl + '] 
          ([MDM_STAGING_ID] INT NOT NULL,
          [MDM_ENTITY_ID] INT NOT NULL,
          [MDM_ENTITY_CLUSTER_ID] INT,
          [MDM_ENTITY_CLUSTER_NAME] NVARCHAR(100) NOT NULL,
          [MATCH_TYPE] INT NOT NULL,
          [OLD_CLUSTER_ID] INT NOT NULL,
          [ROW_PROCESSED] CHAR(1) DEFAULT ''N'' NOT NULL)')
   END
 
END TRY
    BEGIN CATCH
      DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
      SET @error_messages=ERROR_MESSAGE();
      SET @errorSeverity=ERROR_SEVERITY();
      SET @errorState=ERROR_STATE();
      SET @errorStatus=ERROR_NUMBER();
      SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
      set @dttm=getdate()
      SET @mdm_error_messages = 'Location: '+CONVERT(VARCHAR,@err_location)+':'+ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
      --NOTE Do we log old or new id?
      EXEC [mdm_log_error] @parm_entity_type,'mdm_staging_id',@err_staging_id,null,'mdm_createStagingTables',@strErrorStatus,@mdm_error_messages,@dttm
      RAISERROR(@error_messages, @errorSeverity, @errorState)
    END CATCH

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_createStagingIndices]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_createStagingIndices]
GO

CREATE PROCEDURE [mdm_createStagingIndices](@parm_entity_type  VARCHAR(23),
                                       @parm_staging_id  INT = NULL) AS
 DECLARE @err_location INT;
 DECLARE @entitySt_tbl VARCHAR(30);
 DECLARE @entityCc_tbl VARCHAR(30);
 DECLARE @entityTt_tbl VARCHAR(30);
 DECLARE @stg_tbl      VARCHAR(30);
 DECLARE @num_rows     INT;
 DECLARE @tt_num_rows  INT;
 DECLARE @st_num_rows  INT;
 DECLARE @cc_num_rows  INT;
 DECLARE @get_indices CURSOR;
 DECLARE @get_ttIndices CURSOR;
 DECLARE @get_stIndices CURSOR;
 DECLARE @get_ccIndices CURSOR;
 DECLARE @attr_name VARCHAR(50);
 DECLARE @err_staging_id INT;
 DECLARE @tblNameApp VARCHAR(23);
 DECLARE @stgTblNameApp VARCHAR(23);
 DECLARE @constraintName VARCHAR(30);
 DECLARE @indexName      VARCHAR(30);
 DECLARE @tt_index_cols     VARCHAR(500);
 DECLARE @st_index_cols     VARCHAR(500);
 DECLARE @cc_index_cols     VARCHAR(500);
 DECLARE @createIdx      INT;
 DECLARE @stg_idx        INT;
 DECLARE @index_id       INT;
 SET NOCOUNT ON;
BEGIN TRY
 IF (@parm_staging_id IS NULL)
   BEGIN
     SET @tblNameApp = @parm_entity_type
     SET @stgTblNameApp = ''
     SET @entitySt_tbl = 'MDM_'+@parm_entity_type+'_ST'
     SET @entityCc_tbl = 'MDM_'+@parm_entity_type+'_CC'
     SET @stg_tbl = 'MDM_STAGING'
     SET @err_staging_id = 0
   END
 ELSE
   BEGIN
     SET @tblNameApp = 'STG'+CONVERT(VARCHAR,@parm_staging_id)
     SET @stgTblNameApp = '_STG'+CONVERT(VARCHAR,@parm_staging_id)
     SET @entitySt_tbl = 'MDM_BULK_ST_'+CONVERT(VARCHAR,@parm_staging_id)
     SET @entityCc_tbl = 'MDM_BULK_CC_'+CONVERT(VARCHAR,@parm_staging_id)
     SET @stg_tbl = 'MDM_STAGING_' +CONVERT(VARCHAR,@parm_staging_id)
     SET @err_staging_id = @parm_staging_id
   END
 SET @entityTt_tbl = 'MDM_'+@parm_entity_type+'_TT'
   
 -- Add st constraints
 SET @err_location = 10;
 SET @constraintName = 'FKL_ST_'+@tblNameApp
 SELECT @num_rows = COUNT(1) FROM sys.objects WHERE parent_object_id = object_id(@entitySt_tbl) AND lower(name) = lower(@constraintName)
 IF (@num_rows = 0)
   EXEC ('ALTER TABLE [' + @entitySt_tbl + '] ADD CONSTRAINT '+@constraintName+' FOREIGN KEY(MDM_LANGUAGE_ID) 
          REFERENCES MDM_LANGUAGES(MDM_LANGUAGE_ID)')
  
 -- Add st indices
 SET @err_location = 20;
 SET @indexName = 'IX_CID_'+@tblNameApp;
 SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@entitySt_tbl) AND lower(name) = lower(@indexName)
 IF (@num_rows = 0)
   EXEC ('CREATE INDEX '+@indexName+' ON '+ @entitySt_tbl +' (CID)')
   
 SET @err_location = 30;
 SET @indexName = 'ST_MT_'+@tblNameApp;
 SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@entitySt_tbl) AND lower(name) = lower(@indexName)
 IF (@num_rows = 0)
   EXEC ('CREATE INDEX '+@indexName+' ON '+ @entitySt_tbl +' (MDM_STAGING_ID,MATCH_TYPE)')
   
 SET @err_location = 50;
 SET @indexName = 'ST_RID_'+@tblNameApp;
 SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@entitySt_tbl) AND lower(name) = lower(@indexName)
 IF (@num_rows = 0)
   EXEC ('CREATE INDEX '+@indexName+' ON '+ @entitySt_tbl +' (ROW_ID)')
      
 -- Add staging indices
 SET @err_location = 60;
 SET @indexName = 'SG_CID'+@stgTblNameApp;
 SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@stg_tbl) AND lower(name) = lower(@indexName)
 IF (@num_rows = 0)
   EXEC ('CREATE INDEX '+@indexName+' ON '+ @stg_tbl +' (MDM_ENTITY_CLUSTER_ID)')
   
 SET @err_location = 70;
 SET @indexName = 'SG_MT'+@stgTblNameApp;
 SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@stg_tbl) AND lower(name) = lower(@indexName)
 IF (@num_rows = 0)
   EXEC ('CREATE INDEX '+@indexName+' ON '+ @stg_tbl +' (MDM_STAGING_ID,MATCH_TYPE)')
   
 SET @err_location = 80;
 SET @indexName = 'SG_EID'+@stgTblNameApp;
 SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@stg_tbl) AND lower(name) = lower(@indexName)
 IF (@num_rows = 0)   
     EXEC ('CREATE INDEX '+@indexName+' ON '+ @stg_tbl +' (MDM_ENTITY_ID)')
 
 -- Drop any existing entity indices
 SET @get_ttIndices = CURSOR FAST_FORWARD FOR SELECT name FROM sys.indexes 
                              WHERE object_id = object_id(@entityTt_tbl) 
                                AND lower(name) like lower('IDT_'+@parm_entity_type+'_%');
 IF (@parm_staging_id IS NULL)
   BEGIN
     OPEN @get_ttIndices;
       FETCH NEXT FROM @get_ttIndices INTO @indexName;
       WHILE @@FETCH_STATUS=0 
         BEGIN
           EXEC('DROP INDEX '+@indexName+' ON '+@entityTt_tbl)
         FETCH NEXT FROM @get_ttIndices INTO @indexName;
         END 
     CLOSE @get_ttIndices;
     DEALLOCATE @get_ttIndices;     
   END
   
 SET @get_stIndices = CURSOR FAST_FORWARD FOR SELECT name FROM sys.indexes 
                              WHERE object_id = object_id(@entitySt_tbl) 
                                AND lower(name) like lower('IDS_'+@parm_entity_type+'_%');
 OPEN @get_stIndices;
       FETCH NEXT FROM @get_stIndices INTO @indexName;
       WHILE @@FETCH_STATUS=0 
         BEGIN
           EXEC('DROP INDEX '+@indexName+' ON '+@entitySt_tbl)
         FETCH NEXT FROM @get_stIndices INTO @indexName;
         END 
     CLOSE @get_stIndices;
     DEALLOCATE @get_stIndices; 
     
 SET @get_ccIndices = CURSOR FAST_FORWARD FOR SELECT name FROM sys.indexes 
                              WHERE object_id = object_id(@entityCc_tbl) 
                                AND lower(name) like lower('IDC_'+@parm_entity_type+'_%');
 OPEN @get_ccIndices;
       FETCH NEXT FROM @get_ccIndices INTO @indexName;
       WHILE @@FETCH_STATUS=0 
         BEGIN
           EXEC('DROP INDEX '+@indexName+' ON '+@entityCc_tbl)
         FETCH NEXT FROM @get_ccIndices INTO @indexName;
         END 
     CLOSE @get_ccIndices;
     DEALLOCATE @get_ccIndices; 
      
 -- Add entity indices
 SET @get_indices=CURSOR FAST_FORWARD FOR SELECT mdm_index_id, mdm_attr_name
                       FROM mdm_entity_type_indices
                      WHERE LOWER(mdm_entity_type) = LOWER(@parm_entity_type)
                   ORDER BY mdm_index_id,mdm_index_column_order;      
      
 SET @err_location = 90;
 SET @tt_index_cols = '';
 SET @st_index_cols = '';
 SET @cc_index_cols = '';
 SET @createIdx = 0;
 OPEN @get_indices;
   FETCH NEXT FROM @get_indices INTO @index_id,@attr_name;
   WHILE @@FETCH_STATUS=0 
      BEGIN
         SELECT @tt_num_rows=count(1) FROM syscolumns 
            WHERE lower(name)=lower(@attr_name) and id=(SELECT id FROM sysobjects WHERE type='U' AND lower(name)=lower(@entityTt_tbl))
         SELECT @st_num_rows=count(1) FROM syscolumns 
            WHERE lower(name)=lower(@attr_name) and id=(SELECT id FROM sysobjects WHERE type='U' AND lower(name)=lower(@entitySt_tbl))
         SELECT @cc_num_rows=count(1) FROM syscolumns 
            WHERE lower(name)=lower(@attr_name) and id=(SELECT id FROM sysobjects WHERE type='U' AND lower(name)=lower(@entityCc_tbl))
            
         IF (@tt_num_rows > 0 OR @st_num_rows > 0 OR @cc_num_rows > 0) 
           BEGIN
             IF (@stg_idx IS NULL) OR (@stg_idx = @index_id)
               SET @createIdx = 0
             ELSE
               SET @createIdx = 1
             END   
            
             IF (@createIdx = 1)
              BEGIN                
                IF (@parm_staging_id IS NULL) 
                  BEGIN
                    SET @err_location = 100;
                    SET @indexName = 'IDT_'+@tblNameApp+'_'+CONVERT(VARCHAR,@stg_idx);
                    SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@entityTt_tbl) AND lower(name) = lower(@indexName)
                    IF (@num_rows = 0)  AND (len(ltrim(rtrim(@tt_index_cols))) > 0)
                      EXEC ('CREATE INDEX '+@indexName+' ON '+@entityTt_tbl+' ('+@tt_index_cols+')')
                  END
                 
                SET @err_location = 110;
                SET @indexName = 'IDS_'+@tblNameApp+'_'+CONVERT(VARCHAR,@stg_idx); 
                SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@entitySt_tbl) AND lower(name) = lower(@indexName) 
                IF (@num_rows = 0) AND (len(ltrim(rtrim(@st_index_cols))) > 0)
                  EXEC ('CREATE INDEX '+@indexName+' ON '+@entitySt_tbl+' ('+@st_index_cols+')')
                
                SET @err_location = 120; 
                SET @indexName = 'IDC_'+@tblNameApp+'_'+CONVERT(VARCHAR,@stg_idx); 
                SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@entityCc_tbl) AND lower(name) = lower(@indexName) 
                IF (@num_rows = 0) AND (len(ltrim(rtrim(@cc_index_cols))) > 0)
                  EXEC ('CREATE INDEX '+@indexName+' ON '+@entityCc_tbl+' ('+@cc_index_cols+')')
                  
                SET @tt_index_cols = '';
                SET @st_index_cols = '';
                SET @cc_index_cols = '';
              END
             
             IF (len(ltrim(rtrim(@tt_index_cols))) > 0)
               SET @tt_index_cols = @tt_index_cols+','
             IF (len(ltrim(rtrim(@st_index_cols))) > 0)
               SET @st_index_cols = @st_index_cols+','
             IF (len(ltrim(rtrim(@cc_index_cols))) > 0)
               SET @cc_index_cols = @cc_index_cols+','
             
             SET @stg_idx = @index_id;
             
             IF (@tt_num_rows > 0)
               SET @tt_index_cols = @tt_index_cols+@attr_name 
             IF (@st_num_rows > 0)
               SET @st_index_cols = @st_index_cols+@attr_name
             IF (@cc_num_rows > 0)
               SET @cc_index_cols = @cc_index_cols+@attr_name
                 
   FETCH NEXT FROM @get_indices INTO @index_id,@attr_name;                              
   END
 CLOSE @get_indices;
 DEALLOCATE @get_indices;       
       
 IF (len(ltrim(rtrim(@tt_index_cols))) > 0) OR (len(ltrim(rtrim(@st_index_cols))) > 0) OR (len(ltrim(rtrim(@cc_index_cols))) > 0)
    BEGIN
      IF (@parm_staging_id IS NULL) 
         BEGIN
           SET @err_location = 130; 
           SET @indexName = 'IDT_'+@tblNameApp+'_'+CONVERT(VARCHAR,@index_id);
           SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@entityTt_tbl) AND lower(name) = lower(@indexName)
           IF (@num_rows = 0)  AND (len(ltrim(rtrim(@tt_index_cols))) > 0)
              EXEC ('CREATE INDEX '+@indexName+' ON '+@entityTt_tbl+' ('+@tt_index_cols+')')
         END
      
      SET @err_location = 140;             
      SET @indexName = 'IDS_'+@tblNameApp+'_'+CONVERT(VARCHAR,@index_id); 
      SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@entitySt_tbl) AND lower(name) = lower(@indexName) 
      IF (@num_rows = 0) AND (len(ltrim(rtrim(@st_index_cols))) > 0)
         EXEC ('CREATE INDEX '+@indexName+' ON '+@entitySt_tbl+' ('+@st_index_cols+')')
      
      SET @err_location = 150;     
      SET @indexName = 'IDC_'+@tblNameApp+'_'+CONVERT(VARCHAR,@index_id); 
      SELECT @num_rows = COUNT(1) FROM sys.indexes WHERE object_id = object_id(@entityCc_tbl) AND lower(name) = lower(@indexName) 
      IF (@num_rows = 0) AND (len(ltrim(rtrim(@cc_index_cols))) > 0)
         EXEC ('CREATE INDEX '+@indexName+' ON '+@entityCc_tbl+' ('+@cc_index_cols+')')
    END
 
END TRY
    BEGIN CATCH
      DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
      SET @error_messages=ERROR_MESSAGE();
      SET @errorSeverity=ERROR_SEVERITY();
      SET @errorState=ERROR_STATE();
      SET @errorStatus=ERROR_NUMBER();
      SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
      set @dttm=getdate()
      SET @mdm_error_messages = 'Location: '+CONVERT(VARCHAR,@err_location)+':'+ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
      --NOTE Do we log old or new id?
      EXEC [mdm_log_error] @parm_entity_type,'mdm_staging_id',@err_staging_id,null,'mdm_createStagingIndices',@strErrorStatus,@mdm_error_messages,@dttm
      RAISERROR(@error_messages, @errorSeverity, @errorState)
    END CATCH
   
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_alterStTtTables]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_alterStTtTables]
GO

CREATE PROCEDURE [mdm_alterStTtTables](@parm_entity_type_id  INT,
                                       @parm_entity_type  VARCHAR(23),
                                       @parm_is_abstract  CHAR(1)) AS

 DECLARE @num_rows int;
 DECLARE @attr_id int;
 DECLARE @attr_name varchar(50);
 DECLARE @attr_datatype varchar(50);
 DECLARE @attr_nullable varchar(10);
 DECLARE @attr_defn varchar(100);
 DECLARE @attr_length int;
 DECLARE @attr_encrypt char(1);
 DECLARE @parent_id int;
 DECLARE @changeFlag int;
 DECLARE @tt_columns nvarchar(max);
 DECLARE @db_datatype varchar(50);
 DECLARE @tt_name varchar(50);
 DECLARE @strSql nvarchar(max);
 DECLARE @view_name varchar(50);
 DECLARE @get_attrs cursor; 
 
 SET NOCOUNT ON  
BEGIN TRY
   SET @tt_name = 'MDM_' + @parm_entity_type + '_TT';
   SET @changeFlag = 0;
     
   SET @get_attrs=CURSOR FAST_FORWARD FOR WITH tree (mdm_entity_type_id, mdm_parent_id, mdm_label_attr_id) AS 
        (SELECT et.mdm_entity_type_id, et.mdm_parent_id, et.mdm_label_attr_id
        FROM mdm_entity_types et 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
          AND et.MDM_ENTITY_TYPE_ID = @parm_entity_type_id 
        UNION ALL 
        SELECT et.mdm_entity_type_id, et.mdm_parent_id, et.mdm_label_attr_id
        FROM mdm_entity_types et, tree t 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
          AND et.MDM_ENTITY_TYPE_ID = t.mdm_parent_id)

      select a.mdm_attr_id,a.mdm_attr_name,a.mdm_attr_length,a.mdm_encrypt_flag,a.mdm_parent_attr_id,t.mdm_db_datatype,
        case 
          when (t.mdm_db_datatype='VARCHAR' or t.mdm_db_datatype = 'NVARCHAR') then t.mdm_db_datatype+'('+convert(varchar, case a.mdm_attr_length when 0 then 1 else a.mdm_attr_length end)+')'
          else t.mdm_db_datatype end as datatype,
        case 
          when a.mdm_required='Y' 
		  or a.mdm_attr_id IN (select mdm_label_attr_id from tree)
		  then ' NOT NULL CHECK (len('+a.mdm_attr_name+')>=1)'
          else '' end as nullable
      from mdm_attrs a, mdm_attr_datatypes t 
      where a.mdm_attr_id > 0 
        and a.mdm_attr_datatype_id=t.mdm_attr_datatype_id and a.mdm_entity_type_id in (select mdm_entity_type_id from tree)
        and (a.mdm_valid_from_dttm is null or a.mdm_valid_from_dttm <= getdate()) 
        and (a.mdm_valid_to_dttm > getdate() or a.mdm_valid_to_dttm is null)
      order by a.mdm_attr_name;
      
      -- Add columns
      set @tt_columns=''
      OPEN @get_attrs;
      FETCH next from @get_attrs into @attr_id,@attr_name,@attr_length,@attr_encrypt,@parent_id,@db_datatype,@attr_datatype, @attr_nullable;
      WHILE @@FETCH_STATUS=0 BEGIN
        SET @tt_columns = @tt_columns+','+@attr_name
        
        IF (@parm_is_abstract = 'N') BEGIN
           SELECT @num_rows=count(1) FROM syscolumns 
             WHERE lower(name)=lower(@attr_name) and id=(SELECT id FROM sysobjects WHERE type='U' and lower(name)=lower(@tt_name))
           IF (@attr_encrypt='Y' and @parent_id is null) 
             BEGIN          
              SET @attr_datatype = 'VARBINARY(1600)'
             END
		      
           SET @attr_defn=upper(@attr_name)+' '+@attr_datatype+' '+@attr_nullable;      
		      
           IF (@num_rows=0) BEGIN
              SET @changeFlag=1;
              EXEC('ALTER TABLE ['+@tt_name+'] ADD '+@attr_defn);
              UPDATE mdm_attrs SET mdm_valid_from_dttm=getdate() WHERE mdm_attr_id=@attr_id AND mdm_valid_from_dttm IS NULL;
           END;
           
        END;
        FETCH NEXT FROM @get_attrs INTO @attr_id,@attr_name,@attr_length,@attr_encrypt,@parent_id,@db_datatype,@attr_datatype, @attr_nullable;
      END;
      CLOSE @get_attrs;
      DEALLOCATE @get_attrs;
      
      -- Create Staging Tables
      IF (@parm_is_abstract = 'N')
        BEGIN
          EXEC [mdm_createStagingTables] @parm_entity_type_id,@parm_entity_type
          EXEC [mdm_createStagingIndices] @parm_entity_type
        END         
      
      -- Build views
      IF (@changeFlag > 0) OR (@parm_is_abstract = 'Y') BEGIN
         IF (@parm_is_abstract = 'N') BEGIN
		     -- update the views as needed       
	         SET @view_name = 'MDM_' +@parm_entity_type + '_MA';
	         IF EXISTS (SELECT * FROM sys.objects WHERE name = @view_name AND type = 'V')
	            EXEC ('DROP VIEW [' + @view_name + ']')
	         SET @strSQL = N'CREATE VIEW ['+ @view_name + ']  AS 
	            SELECT T.*,ET.MDM_ENTITY_TYPE_ID
	              FROM [' +@tt_name+ '] T
	             INNER JOIN MDM_ENTITY_TYPES ET ON LOWER(ET.MDM_ENTITY_TYPE) = LOWER('''+@parm_entity_type+''')
	             WHERE T.MDM_SURVIVOR = ''N''
	               AND T.MDM_VALID_FROM_DTTM <= getdate() and (T.MDM_VALID_TO_DTTM > GETDATE() OR T.MDM_VALID_TO_DTTM IS NULL)';
	         EXEC (@strSQL)
	
	         SET @view_name = 'MDM_' + @parm_entity_type + '_CV';
	         IF EXISTS (SELECT * FROM sys.objects WHERE name = @view_name and type = 'V')
	            EXEC ('DROP VIEW [' + @view_name + ']')
	         SET @strSQL = N'CREATE VIEW ['+ @view_name + ']  AS 
	            SELECT T.*,ET.MDM_ENTITY_TYPE_ID
	              FROM [' +@tt_name+ '] T
	             INNER JOIN MDM_ENTITY_TYPES ET ON LOWER(ET.MDM_ENTITY_TYPE) = LOWER('''+@parm_entity_type+''')
	             WHERE T.MDM_VALID_FROM_DTTM <= getdate() and (T.MDM_VALID_TO_DTTM > GETDATE() OR T.MDM_VALID_TO_DTTM IS NULL)';
	         EXEC (@strSQL)
         END
     
		 EXEC [mdm_buildSearchVw] @parm_entity_type_id,@parm_entity_type,@parm_is_abstract
	  END;    
END TRY
    BEGIN CATCH
      DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
      SET @error_messages=ERROR_MESSAGE();
      SET @errorSeverity=ERROR_SEVERITY();
      SET @errorState=ERROR_STATE();
      SET @errorStatus=ERROR_NUMBER();
      SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
      set @dttm=getdate()
      SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
      --NOTE Do we log old or new id?
      EXEC [mdm_log_error] 'MDM_ENTITY_TYPES',null,@parm_entity_type_id,'mdm_entity_type_id','mdm_alterStTtTables',@strErrorStatus,@mdm_error_messages,@dttm
      RAISERROR(@error_messages, @errorSeverity, @errorState)
    END CATCH
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_entityType_goLive]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_entityType_goLive]
GO

create PROCEDURE [mdm_entityType_goLive](@parm_entity_type_id INT,
                                         @parm_opseq  INT) AS
  declare @tt_name varchar(50);
  declare @sv_name varchar(50);
  declare @al_name varchar(50);
  declare @num_rows int;
  declare @changeFlag int;
  declare @strSql nvarchar(max);
  declare @strEntitySql nvarchar(max);
  declare @strClusterSql nvarchar(max);
  declare @view_name varchar(50);
  declare @activeType varchar(23);
  declare @activeTypeId int;
  declare @mdm_entity_type varchar(23);
  declare @opseq int;
  declare @get_parentClasses cursor;
  declare @get_subClasses cursor;
  declare @get_activeEntitytypes cursor;
  declare @parent_type varchar(20);
  declare @parent_type_id int;
  declare @isParentAbstract char(1);
  declare @childId int;
  declare @childType varchar(23);
  declare @isChildAbstract char(1);
  declare @parent_attr varchar(50);
  declare @isAbstract char(1);
  
  BEGIN
    SET NOCOUNT ON    
    begin try
    
	-- make sure we have an entity type to process.
	SELECT @mdm_entity_type=upper(t.mdm_entity_type), @opseq = mdm_opseq, @isAbstract = mdm_abstract FROM mdm_entity_types t WHERE t.mdm_entity_type_id=@parm_entity_type_id;
	IF (@mdm_entity_type IS NULL) BEGIN
	  SET @strSQL='Could not make '+CONVERT(varchar(10), @parm_entity_type_id)+' live; the entity type is not defined';
      RAISERROR(@strSQL, 11,1);
      RETURN;
	END;
	
	IF (@parm_opseq <> @opseq) BEGIN
      SET @strSql = 'Entity Type Out of Date';
      RAISERROR(@strSql, 11,1);
      RETURN;
    END
    
    SET @changeFlag=0;
    SET @sv_name = 'MDM_'+@mdm_entity_type+'_SV'
    SELECT @num_rows=count(1) FROM sys.objects WHERE name = @sv_name;
    IF (@num_rows = 0) BEGIN
        SET @changeFlag = 1;
      	
	IF (@isAbstract = 'N') BEGIN	
      
      -- if the transpose table doesn't already exist, create it.
      set @tt_name = 'MDM_' + @mdm_entity_type + '_TT';
      set @al_name = 'MDM_' + @mdm_entity_type + '_AL';
      select @num_rows=count(1) from sys.objects where name = @tt_name;
      IF (@num_rows = 0) begin
        set @changeFlag = 1;
        EXEC ('CREATE TABLE [' + @tt_name + '] 
          ([MDM_STAGING_ID] INT NOT NULL,
          [MDM_ENTITY_ID] INT NOT NULL,
          [MDM_SRC_SYS_ID] INT NOT NULL,
          [MDM_SRC_SYS_REC_ID] NVARCHAR(200) NOT NULL,
          [MDM_LANGUAGE_ID] INT NOT NULL,
          [MDM_ENTITY_CLUSTER_ID] INT NOT NULL,
          [MDM_SURVIVOR] CHAR(1) NOT NULL,
          [MDM_VALID_FROM_DTTM] DATETIME NOT NULL,
          [MDM_VALID_TO_DTTM] DATETIME)')
        EXEC ('ALTER TABLE MDM_'+@mdm_entity_type+'_TT ADD CONSTRAINT PK_'+@mdm_entity_type+'_TT PRIMARY KEY (MDM_ENTITY_ID)')
        EXEC ('ALTER TABLE [' + @tt_name + '] ADD CONSTRAINT FKL_'+@mdm_entity_type+'_TT FOREIGN KEY(MDM_LANGUAGE_ID) 
          REFERENCES MDM_LANGUAGES(MDM_LANGUAGE_ID)')        
      END;
      
      -- if the audit log doesn't already exist, create it.
      select @num_rows=count(1) from sys.objects where name = @al_name;
      IF (@num_rows = 0) begin
        set @changeFlag= 1;
        EXEC ('CREATE TABLE [' + @al_name + '] 
          ([MDM_AUDIT_LOG_ID] INT IDENTITY(1,1),
          [TABLE_NAME] VARCHAR(50) NOT NULL,
          [RECORD_ID] INT NOT NULL,
          [USER_ACTION] VARCHAR(10) NOT NULL,
          [MDM_MODIFIED_BY] NVARCHAR(513) NOT NULL,
          [MODIFIED_DTTM] DATETIME NOT NULL)')
        EXEC ('CREATE INDEX AL_'+ @mdm_entity_type +'_RID ON '+ @al_name +' (RECORD_ID)')
      END;    
           
      END;
      -- process the entity type's attributes
      EXEC [mdm_alterStTtTables] @parm_entity_type_id, @mdm_entity_type, @isAbstract         
     END;
     ELSE
       BEGIN
         SET @get_subclasses=CURSOR FAST_FORWARD FOR WITH tree (mdm_entity_type, mdm_entity_type_id, mdm_abstract, mdm_parent_id) AS 
            (SELECT et.mdm_entity_type, et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id
               FROM mdm_entity_types et 
              WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
                AND et.MDM_ENTITY_TYPE_ID = @parm_entity_type_id 
             UNION ALL 
             SELECT et.mdm_entity_type, et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id 
               FROM mdm_entity_types et, tree t 
              WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null)
                AND et.mdm_valid_from_dttm <= GETDATE()
                AND et.mdm_parent_id = t.mdm_entity_type_id
                AND et.mdm_entity_type_id > 0)
            SELECT mdm_entity_type_id, mdm_entity_type, mdm_abstract FROM tree ORDER BY mdm_abstract;
		 OPEN @get_subClasses;
         FETCH NEXT FROM @get_subClasses INTO @childId,@childType,@isChildAbstract;
         WHILE @@FETCH_STATUS=0 BEGIN
           EXEC [mdm_alterStTtTables] @childId, @childType, @isChildAbstract
         FETCH NEXT FROM @get_subClasses INTO @childId,@childType,@isChildAbstract;
         END	   
    END;
    
    update mdm_entity_types set MDM_GOLIVE_DTTM=getdate(),
      mdm_valid_from_dttm=CASE WHEN mdm_valid_from_dttm is null THEN getdate() ELSE mdm_valid_from_dttm END
      where mdm_entity_type_id=@parm_entity_type_id;
    update mdm_attrs set 
      mdm_valid_from_dttm=CASE WHEN mdm_valid_from_dttm is null THEN getdate() ELSE mdm_valid_from_dttm END
      where mdm_entity_type_id=@parm_entity_type_id;

    SET @get_parentClasses=CURSOR FAST_FORWARD FOR WITH tree (mdm_entity_type, mdm_entity_type_id, mdm_abstract, mdm_parent_id) AS 
      (SELECT et.mdm_entity_type, et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id
      FROM mdm_entity_types et 
      WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
        AND et.MDM_ENTITY_TYPE_ID = @parm_entity_type_id 
      UNION ALL 
      SELECT et.mdm_entity_type, et.mdm_entity_type_id, et.mdm_abstract, et.mdm_parent_id 
      FROM mdm_entity_types et, tree t 
      WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
        AND et.mdm_valid_from_dttm <= GETDATE()
        AND et.mdm_entity_type_id = t.mdm_parent_id)
    SELECT mdm_entity_type_id,mdm_entity_type,mdm_abstract FROM tree WHERE mdm_entity_type_id <> @parm_entity_type_id;
    
    OPEN @get_parentClasses;
     FETCH NEXT FROM @get_parentClasses INTO @parent_type_id,@parent_type,@isParentAbstract;
      WHILE @@FETCH_STATUS=0 BEGIN  
        EXEC [mdm_buildSearchVw] @parent_type_id,@parent_type,@isParentAbstract          
      FETCH NEXT FROM @get_parentClasses INTO @parent_type_id,@parent_type,@isParentAbstract;
      END          
    CLOSE @get_parentClasses
    DEALLOCATE @get_parentClasses
    
    if (@changeFlag=0)
      select mdm_golive_dttm as pubDate,mdm_valid_from_dttm,mdm_last_mod_dttm, 'No changes found' as message from mdm_entity_types where mdm_entity_type_id=@parm_entity_type_id;
    else
      select mdm_golive_dttm as pubDate,mdm_valid_from_dttm,mdm_last_mod_dttm, 'Changes applied' as message from mdm_entity_types where mdm_entity_type_id=@parm_entity_type_id;
     
    end try
    begin catch
      DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
      SET @error_messages=ERROR_MESSAGE();
      SET @errorSeverity=ERROR_SEVERITY();
      SET @errorState=ERROR_STATE();
      SET @errorStatus=ERROR_NUMBER();
      SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
      set @dttm=getdate()
      SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
      --NOTE Do we log old or new id?
      EXEC [mdm_log_error] 'MDM_ENTITYTYPE_GOLIVE',null,@parm_entity_type_id,'mdm_entity_type_id','mdm_entityType_goLive',@strErrorStatus,@mdm_error_messages,@dttm
      RAISERROR(@error_messages, @errorSeverity, @errorState)
    end catch
  END;


GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_encrypt_value]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_encrypt_value]
GO

CREATE PROCEDURE [mdm_encrypt_value] (@parm_value VARCHAR(500)) AS

declare @key uniqueidentifier;
declare @sqlStmt nvarchar(200);
declare @keyName varchar(50);
declare @certName varchar(50);

BEGIN TRY  
    OPEN SYMMETRIC KEY qMDMKey DECRYPTION BY CERTIFICATE qMDMCer
      
    SELECT @key = KEY_GUID('qMDMKey')     
    SELECT ENCRYPTBYKEY(@key,CONVERT(varbinary(256),@parm_value)) enc_value
   
    CLOSE SYMMETRIC KEY qMDMKey
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] null,null,0,null,'mdm_encrypt_value',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_getDecryptTtCols]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_getDecryptTtCols]
GO

CREATE PROCEDURE [mdm_getDecryptTtCols] (@parm_entity_type_id  INTEGER) AS

declare @tt_columns nvarchar(max);
declare @attr_name varchar(50);
declare @encrypt_flag char(1);
declare @parent_id int;
declare @last_attr int;
declare @i int;

BEGIN TRY  
    SET NOCOUNT ON
    --Generate transpose table column string
    CREATE TABLE #attr_tbl (attr_num INT IDENTITY(1,1),attr_name VARCHAR(50),encrypt_flg CHAR(1),parent_id INT);
    
    WITH tree (mdm_entity_type_id, mdm_parent_id) AS 
        (SELECT et.mdm_entity_type_id, et.mdm_parent_id
        FROM mdm_entity_types et 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
          AND et.MDM_ENTITY_TYPE_ID = @parm_entity_type_id 
        UNION ALL 
        SELECT et.mdm_entity_type_id, et.mdm_parent_id 
        FROM mdm_entity_types et, tree t 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
          AND et.MDM_ENTITY_TYPE_ID = t.mdm_parent_id)
    INSERT INTO #attr_tbl (attr_name,encrypt_flg,parent_id)
      SELECT mdm_attr_name,mdm_encrypt_flag,mdm_parent_attr_id FROM mdm_attrs
       WHERE mdm_entity_type_id IN (SELECT mdm_entity_type_id FROM tree)
         AND mdm_valid_from_dttm < GETDATE() and (mdm_valid_to_dttm is null or mdm_valid_to_dttm > GETDATE())
         AND mdm_attr_id > 0;
  
    SELECT @last_attr = MAX(attr_num)
      FROM #attr_tbl;
       
    SET @i = 1;
    SET @tt_columns = '';
     
    WHILE (@i <= @last_attr) 
      BEGIN         
        SELECT @attr_name = attr_name,
               @encrypt_flag = encrypt_flg,
               @parent_id = parent_id
          FROM #attr_tbl
         WHERE attr_num = @i;
         
        IF (@encrypt_flag = 'Y' and @parent_id IS NULL)
          BEGIN
            SET @tt_columns = @tt_columns+',CONVERT(varchar,DECRYPTBYKEY('+@attr_name+')) as '+@attr_name;
          END
        ELSE 
          BEGIN
            SET @tt_columns = @tt_columns+','+@attr_name;
          END
        
        SET @i = @i + 1;
      END   
     
    DROP TABLE #attr_tbl;
    
    SELECT @tt_columns tt_columns
    
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] null,null,0,null,'mdm_getDecryptTtCols',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 


GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_decryptCols]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_decryptCols]
GO

CREATE PROCEDURE [mdm_decryptCols] (@parm_entity_type     VARCHAR(30),
                                    @parm_entity_type_id  INT,
                                    @parm_tbl_suffix      VARCHAR(10)) AS

declare @sqlStmt nvarchar(max);
declare @key uniqueidentifier;
declare @tt_columns nvarchar(max);

BEGIN TRY  
    SET NOCOUNT ON    
        
    CREATE TABLE #colStr (colStrValue nvarchar(max));
    
    INSERT INTO #colStr   
    
    EXEC [mdm_getDecryptTtCols] @parm_entity_type_id    
    SELECT @tt_columns = colStrValue FROM #colStr
    
    DROP TABLE #colStr
    
    OPEN SYMMETRIC KEY qMDMKey DECRYPTION BY CERTIFICATE qMDMCer
      
    SET @sqlStmt = 'select mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,'+
                          'mdm_valid_from_dttm,mdm_valid_to_dttm,mdm_entity_cluster_id,mdm_survivor'+@tt_columns+' '+
                 'from mdm_'+@parm_entity_type+'_'+@parm_tbl_suffix;
                 
    EXECUTE sp_executesql @sqlStmt
    
    CLOSE SYMMETRIC KEY qMDMKey    
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] null,null,0,null,'mdm_decryptCols',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_decryptCols_byClusterId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_decryptCols_byClusterId]
GO

CREATE PROCEDURE [mdm_decryptCols_byClusterId] (@parm_entity_type     VARCHAR(30),
                                                @parm_cluster_id      INT,
                                                @parm_tt_columns      NVARCHAR(MAX),
                                                @parm_tbl_suffix      VARCHAR(10)) AS

declare @sqlStmt nvarchar(max);
declare @entityTypeId int;
declare @key uniqueidentifier;

BEGIN TRY  
    
    SELECT @entityTypeId = mdm_entity_type_id
      FROM mdm_entity_types
     WHERE LOWER(mdm_entity_type) = LOWER(@parm_entity_type)
       and mdm_valid_from_dttm <= GETDATE() 
       and (mdm_valid_to_dttm is null or mdm_valid_to_dttm > GETDATE());
    
    OPEN SYMMETRIC KEY qMDMKey DECRYPTION BY CERTIFICATE qMDMCer
      
    SET @sqlStmt = 'select fc.original_cluster_id,fc.current_cluster_id,ev.mdm_entity_type_id,ev.mdm_entity_id,ev.mdm_src_sys_id,ev.mdm_src_sys_rec_id,ev.mdm_language_id,'+
                    'ev.mdm_valid_from_dttm,ev.mdm_valid_to_dttm,ev.mdm_entity_cluster_id,ev.mdm_survivor'+@parm_tt_columns+' '+
               'from mdm_'+@parm_entity_type+'_'+@parm_tbl_suffix+' ev '+
              'left join mdm_forced_clusters fc on fc.src_sys_id = ev.mdm_src_sys_id and fc.src_sys_rec_id = ev.mdm_src_sys_rec_id and fc.entity_type_id = @entityId '+
              'where ev.mdm_entity_cluster_id = @cluster_id '+
              'order by ev.mdm_survivor desc';
                 
    EXECUTE sp_executesql @sqlStmt,N'@entityId int,@cluster_id int',@entityTypeId,@parm_cluster_id
    
    CLOSE SYMMETRIC KEY qMDMKey    
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] null,null,0,null,'mdm_decryptCols_byClusterId',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_decryptCols_byEntityId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_decryptCols_byEntityId]
GO

CREATE PROCEDURE [mdm_decryptCols_byEntityId] (@parm_entity_type     VARCHAR(30),
                                               @parm_entity_id      INT,
                                               @parm_tt_columns      NVARCHAR(MAX),
                                               @parm_tbl_suffix      VARCHAR(10)) AS

declare @sqlStmt nvarchar(max);
declare @key uniqueidentifier;

BEGIN TRY  
    
    OPEN SYMMETRIC KEY qMDMKey DECRYPTION BY CERTIFICATE qMDMCer
      
    SET @sqlStmt = 'select mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,'+
                          'mdm_valid_from_dttm,mdm_valid_to_dttm,mdm_entity_cluster_id,mdm_survivor'+@parm_tt_columns+' '+
                 'from mdm_'+@parm_entity_type+'_'+@parm_tbl_suffix+' '+
                'where mdm_entity_id = @entity_id';
                 
    EXECUTE sp_executesql @sqlStmt,N'@entity_id int',@parm_entity_id
    
    CLOSE SYMMETRIC KEY qMDMKey    
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] null,null,0,null,'mdm_decryptCols_byEntityId',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_getDecryptString]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_getDecryptString]
GO

CREATE PROCEDURE [mdm_getDecryptString] (@parm_attr_name     VARCHAR(50)) AS

BEGIN   
    
    SELECT 'CONVERT(varchar,DECRYPTBYKEY('+@parm_attr_name+'))' AS attr_string;                 
       
END 
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_queryCluster]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_queryCluster]
GO

CREATE PROCEDURE [mdm_queryCluster](@parm_entity_type VARCHAR(30),
									@parm_cluster_id INT,
									@parm_show_retired CHAR(1)) AS

 DECLARE @sqlStmt NVARCHAR(MAX);
 DECLARE @tt_columns NVARCHAR(MAX);	
 DECLARE @entityTypeId INT;
 SET NOCOUNT ON
 BEGIN TRY
    SELECT @entityTypeId = mdm_entity_type_id FROM mdm_entity_types WHERE lower(mdm_entity_type) = lower(@parm_entity_type)
	CREATE TABLE #colStr (colStrValue nvarchar(max));
    INSERT INTO #colStr   
    EXEC [mdm_getDecryptTtCols] @entityTypeId    
    SELECT @tt_columns = colStrValue FROM #colStr
    
    DROP TABLE #colStr
    
    OPEN SYMMETRIC KEY qMDMKey DECRYPTION BY CERTIFICATE qMDMCer
    
    IF (@parm_show_retired = 'Y')
      BEGIN
        SET @sqlStmt = 'select * from (select '+CONVERT(VARCHAR,@entityTypeId)+' as mdm_entity_type_id,v.mdm_entity_id,v.mdm_src_sys_id,v.mdm_src_sys_rec_id,v.mdm_language_id,'+
                         'v.mdm_valid_from_dttm,v.mdm_valid_to_dttm,v.mdm_entity_cluster_id,v.mdm_survivor'+@tt_columns+','+
                         'null as mod_dttm,h.mdm_comment, h.mdm_reason from mdm_'+@parm_entity_type+'_tt v '+
                    'left join mdm_cluster_member_history h on v.mdm_entity_id=h.mdm_entity_id '+
                    'where v.mdm_entity_cluster_id=@cluster_id and (h.mdm_valid_to_dttm > getdate() or h.mdm_valid_to_dttm is null) '+  
                  'union '+
                  'select '+CONVERT(VARCHAR,@entityTypeId)+' as mdm_entity_type_id,v.mdm_entity_id,v.mdm_src_sys_id,v.mdm_src_sys_rec_id,v.mdm_language_id,'+ 
                         'v.mdm_valid_from_dttm,v.mdm_valid_to_dttm,v.mdm_entity_cluster_id,v.mdm_survivor'+@tt_columns+','+
                         'h.mdm_modified_dttm as mod_dttm,h.mdm_comment,h.mdm_reason '+
                         'from mdm_'+@parm_entity_type+'_tt v, mdm_cluster_member_history h '+
                   'where h.mdm_entity_cluster_id=@cluster_id and h.mdm_entity_id=v.mdm_entity_id ) e '+
                 'order by mdm_valid_from_dttm desc,mdm_src_sys_id asc';   
      END
    ELSE
      BEGIN
        SET @sqlStmt = 'select '+CONVERT(VARCHAR,@entityTypeId)+' as mdm_entity_type_id,v.mdm_entity_id,v.mdm_src_sys_id,v.mdm_src_sys_rec_id,v.mdm_language_id,'+
                          'v.mdm_valid_from_dttm,v.mdm_valid_to_dttm,v.mdm_entity_cluster_id,v.mdm_survivor'+@tt_columns+','+
                          'h.mdm_comment,h.mdm_reason,h.mdm_modified_dttm as mod_dttm from mdm_'+@parm_entity_type+'_tt v '+
                    'left join mdm_cluster_member_history h on v.mdm_entity_id=h.mdm_entity_id '+
                   'where v.mdm_entity_cluster_id = @cluster_id '+
                     'and v.mdm_valid_from_dttm <= getdate() '+
                     'and (v.mdm_valid_to_dttm is null or v.mdm_valid_to_dttm > getdate()) '+
                  'order by v.mdm_valid_from_dttm desc,v.mdm_src_sys_id asc';
      END
      
      PRINT (@sqlStmt);
      
      EXECUTE sp_executesql @sqlStmt,N'@cluster_id int',@parm_cluster_id 
      
      CLOSE SYMMETRIC KEY qMDMKey    
 END TRY
 BEGIN CATCH		
	DECLARE @error_messages VARCHAR(2000)
	DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

	SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									' : '+@errorSeverityString+
									' : '+@mdm_error_status+
									' : '+@errorStateString
	-- Log Error
	EXEC [mdm_log_error] null,null,@parm_cluster_id,'mdm_cluster_id','mdm_queryCluster',@mdm_error_status,@mdm_error_messages,@errorDate
	
	-- Raise Error	
	RAISERROR(@error_messages, @errorSeverity, @errorState)
 END CATCH	


GO


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_getAttrs]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_getAttrs]
GO

CREATE PROCEDURE [mdm_getAttrs] (@parm_entity_type  VARCHAR(23),
                                 @parm_attr_type    CHAR(1)) AS

DECLARE @sqlStmt NVARCHAR(1000);
DECLARE @typeClause VARCHAR(200);

BEGIN  
   BEGIN TRY
     
     IF (@parm_attr_type = 'U') 
       BEGIN
          SET @typeClause = 'and mdm_parent_attr_id is null ';
       END;
     ELSE IF (@parm_attr_type = 'D')
            BEGIN
               SET @typeClause = ' ';
            END
          ELSE IF (@parm_attr_type = 'R')
                 BEGIN
					SET @typeClause = 'and mdm_parent_attr_id is null and (mdm_required=''Y'' or (mdm_attr_id IN (select mdm_label_attr_id from tree)))';
                 END
               ELSE IF (@parm_attr_type = 'E')
                      BEGIN
                         SET @typeClause = 'and mdm_parent_attr_id is null and mdm_encrypt_flag = ''Y'' ';
                      END
                    ELSE
                      BEGIN
                         SET @typeClause = ' ';
                      END
     
     SET @sqlStmt = ';WITH tree (mdm_entity_type_id, mdm_parent_id, mdm_label_attr_id) AS '+
						'(SELECT mdm_entity_type_id, mdm_parent_id, mdm_label_attr_id '+
                           'FROM mdm_entity_types et '+ 
                           'WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) '+
                             'AND lower(et.MDM_ENTITY_TYPE) = lower(@parm_entity_type) '+     
                         'UNION ALL '+
                         'SELECT et.mdm_entity_type_id, et.mdm_parent_id, et.mdm_label_attr_id '+
                           'FROM mdm_entity_types et, tree t '+
                          'WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) '+
                            'AND et.MDM_ENTITY_TYPE_ID = t.mdm_parent_id ) '+
                    'SELECT mdm_attr_id, mdm_attr_name '+
                      'FROM mdm_attrs '+
                     'WHERE mdm_entity_type_id in (select mdm_entity_type_id from tree ) '+
                       'AND (mdm_valid_to_dttm > GETDATE() or mdm_valid_to_dttm is null) '+@typeClause+' '+                       
                      'ORDER BY mdm_attr_id '
                      
     EXECUTE SP_Executesql @sqlStmt,N'@parm_entity_type  VARCHAR(23)', @parm_entity_type
                      
   END TRY
   BEGIN CATCH		
		DECLARE @error_messages VARCHAR(2000)
		DECLARE @errorSeverity INT
		DECLARE @errorState  INT
		DECLARE @errorStatus INT
		DECLARE @errorProc  VARCHAR(150)
		DECLARE @errorDate DATETIME

		DECLARE @mdm_error_status  VARCHAR(10)
		DECLARE @mdm_error_messages VARCHAR(4000)
		DECLARE @errorSeverityString VARCHAR(100)
		DECLARE @errorStateString VARCHAR(100)

		SELECT @errorStatus = ERROR_NUMBER()
		SELECT @error_messages = ERROR_MESSAGE()
		SELECT @errorSeverity = ERROR_SEVERITY()
		SELECT @errorState  = ERROR_STATE()
		SELECT @errorProc = ERROR_PROCEDURE()
		SELECT @errorDate = getDate()

		SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
		SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
		SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)

		SELECT @mdm_error_messages = @errorProc+ ' : '+@error_messages+ 
									' : '+@errorSeverityString+
									' : '+@mdm_error_status+
									' : '+@errorStateString
		-- Log Error
		EXEC [mdm_log_error] 'MDM_ENTITY_TYPES',null,null,null,'mdm_getAttrs',@mdm_error_status,@mdm_error_messages,@errorDate

		
		-- Raise Error	
		RAISERROR(@error_messages, @errorSeverity, @errorState)
	END CATCH
 END
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_retireEntityType]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_retireEntityType]
GO

CREATE PROCEDURE [mdm_retireEntityType]  (@parm_entity_type_id INT) AS
  DECLARE @retire_dttm DATETIME;
  DECLARE @message VARCHAR(100);
  DECLARE @err_location INT;
SET NOCOUNT ON;
BEGIN TRY  
    SELECT @retire_dttm = GETDATE();
    
    CREATE TABLE #withSubclasses (entity_type_id INT,is_abstract CHAR(1));
    
    SET @err_location = 10;
    
    WITH tree (mdm_entity_type_id) AS 
        (SELECT et.mdm_entity_type_id
        FROM mdm_entity_types et 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null) 
          AND et.MDM_ENTITY_TYPE_ID = @parm_entity_type_id
        UNION ALL 
        SELECT et.mdm_entity_type_id
        FROM mdm_entity_types et, tree t 
        WHERE (et.mdm_valid_to_dttm > GETDATE() or et.mdm_valid_to_dttm is null)
          AND et.mdm_parent_id = t.mdm_entity_type_id
          AND et.mdm_entity_type_id > 0)
    INSERT INTO #withSubclasses (entity_type_id) SELECT mdm_entity_type_id from tree
      
    SET @err_location = 20;
      
    BEGIN TRAN
    	SET @err_location = 30;
	    -- Retire attributes     
    	UPDATE mdm_attrs SET mdm_valid_to_dttm = @retire_dttm
		WHERE mdm_entity_type_id IN (SELECT entity_type_id FROM #withSubclasses);

		UPDATE mdm_attr_access SET mdm_active = 'N'
		WHERE mdm_entity_type_id IN (SELECT entity_type_id FROM #withSubclasses);

		--Cluster conditions
		UPDATE mdm_cluster_conditions SET mdm_valid_to_dttm = @retire_dttm
		WHERE mdm_entity_type_id IN (SELECT entity_type_id FROM #withSubclasses);

		--Relationship types
		UPDATE mdm_entity_rel_types SET mdm_valid_to_dttm = @retire_dttm
		WHERE mdm_from_entity_type_id IN (SELECT entity_type_id FROM #withSubclasses)
		  or mdm_to_entity_type_id IN (SELECT entity_type_id FROM #withSubclasses);

		-- Retire entity type roles        
		UPDATE mdm_entity_type_access SET mdm_active = 'N'
		WHERE mdm_entity_type_id IN (SELECT entity_type_id FROM #withSubclasses);
         
		-- Finally retire entity type      
		UPDATE mdm_entity_types SET mdm_valid_to_dttm = @retire_dttm
		WHERE mdm_entity_type_id IN (SELECT entity_type_id FROM #withSubclasses);     
     
		SET @message = ''  
    COMMIT TRAN
    
    DROP TABLE #withSubclasses;
    SET @err_location = 40;
    SELECT mdm_last_mod_dttm, mdm_valid_to_dttm, @message as message FROM mdm_entity_types WHERE mdm_entity_type_id = @parm_entity_type_id
END TRY
BEGIN CATCH
	ROLLBACK TRAN
	DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
	SET @error_messages=ERROR_MESSAGE();
	SET @errorSeverity=ERROR_SEVERITY();
	SET @errorState=ERROR_STATE();
	SET @errorStatus=ERROR_NUMBER();
	SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
	set @dttm=getdate()
	SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
	EXEC [mdm_log_error] 'MDM_ENTITY_TYPES',null,@parm_entity_type_id,'mdm_entity_type_id','mdm_retireEntityType',@errorStatus,@mdm_error_messages,@dttm
	RAISERROR(@error_messages, @errorSeverity, @errorState)
END CATCH
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_createColumns]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_createColumns]
GO

CREATE PROCEDURE [mdm_createColumns](@parm_table_name  VARCHAR(30),
                                     @parm_column_name  VARCHAR(30),
                                     @parm_datatype  VARCHAR(30),
                                     @parm_length    INT,
                                     @parm_locale    VARCHAR(5)) AS

 DECLARE @dbDatatype  VARCHAR(30);
 DECLARE @colName     VARCHAR(35);
 DECLARE @colType     VARCHAR(50);
 DECLARE @num_rows    INT;
 
 SET NOCOUNT ON  
BEGIN TRY
   SELECT @dbDatatype = mdm_db_datatype FROM mdm_attr_datatypes
    WHERE lower(mdm_attr_datatype) = lower(@parm_datatype)
    
   SET @colName = @parm_column_name+'_'+@parm_locale
   
   IF (lower(@dbDatatype) = 'varchar' OR lower(@dbDatatype) = 'nvarchar') 
    SET @colType = @dbDatatype+'('+CONVERT(VARCHAR,@parm_length)+')'    
   ELSE
    SET @colType = @dbDatatype; 

   SELECT @num_rows=count(1) FROM syscolumns 
    WHERE lower(name)=lower(@colName) and id=(SELECT id FROM sysobjects WHERE type='U' AND lower(name)=lower(@parm_table_name))
   IF (@num_rows=0) 
    EXEC('ALTER TABLE ['+@parm_table_name+'] ADD '+@colName+' '+@colType);           
END TRY
    BEGIN CATCH
      DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
      SET @error_messages=ERROR_MESSAGE();
      SET @errorSeverity=ERROR_SEVERITY();
      SET @errorState=ERROR_STATE();
      SET @errorStatus=ERROR_NUMBER();
      SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
      set @dttm=getdate()
      SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
      EXEC [mdm_log_error] @parm_table_name,null,0,null,'mdm_createColumns',@strErrorStatus,@mdm_error_messages,@dttm
      RAISERROR(@error_messages, @errorSeverity, @errorState)
    END CATCH

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_entityIdExists]') AND type in (N'P', N'PC'))
DROP PROCEDURE mdm_entityIdExists
GO

CREATE PROCEDURE mdm_entityIdExists (@parm_entity_type     VARCHAR(30),
                                     @parm_entity_id      INT) AS

SET NOCOUNT ON
declare @sqlStmt nvarchar(100);
declare @recordExists  int;

BEGIN TRY  
    
    SET @sqlStmt = 'select @recordExists = 1 from mdm_'+@parm_entity_type+'_tt where mdm_entity_id = @entId';                 
    EXECUTE sp_executesql @sqlStmt,N'@entId int,@recordExists INTEGER OUTPUT',@parm_entity_id,@recordExists OUTPUT
    
    IF @recordExists IS NULL 
      SET @recordExists = 0;
      
    SELECT @recordExists AS record_exists
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] @parm_entity_id,null,0,null,'mdm_entityIdExists',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_dashboardCountsByDate]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_dashboardCountsByDate]
GO

CREATE PROCEDURE [mdm_dashboardCountsByDate] (@parm_entity_type     VARCHAR(30)) AS

SET NOCOUNT ON
declare @strSql nvarchar(1000);

BEGIN TRY  
    
    SET @strSql = 'SELECT COUNT(*) records, mdm_src_sys_id, status, dtquery.mdm_valid_from_dttm, dtquery.mdm_valid_to_dttm
                     FROM (SELECT mdm_src_sys_id, mdm_valid_from_dttm, mdm_valid_to_dttm, 
                                  CASE WHEN mdm_valid_to_dttm IS NULL THEN ''ACTIVE'' ELSE ''RETIRED''
                                      END AS STATUS
                             FROM mdm_'+@parm_entity_type+'_tt) dtquery 
                    GROUP BY  mdm_src_sys_id, status,dtquery.mdm_valid_from_dttm,dtquery.mdm_valid_to_dttm';                 
    EXECUTE sp_executesql @strSql
        
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] @parm_entity_type,null,0,null,'mdm_dashboardCountsByDate',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_dashboardGetActiveRetired]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_dashboardGetActiveRetired]
GO

CREATE PROCEDURE [mdm_dashboardGetActiveRetired] (@parm_entity_type     VARCHAR(30),
                                                  @parm_curr_dttm       DATETIME,
                                                  @parm_active_retired  VARCHAR(10)) AS

SET NOCOUNT ON
DECLARE @strSql nvarchar(1000);
DECLARE @field varchar(50);
BEGIN TRY  
    IF (@parm_active_retired = 'ACTIVE') 
      SET @field = 'mdm_valid_from_dttm'
    ELSE IF (@parm_active_retired = 'RETIRED')
            SET @field = 'mdm_valid_to_dttm'         
    
    SET @strSql = 'SELECT COUNT(*) as records,mdm_src_sys_id, addrange
                 FROM (SELECT mdm_src_sys_id,
                              CASE WHEN '+@field+' >= (@dttm-1) THEN ''DAY''
                                   WHEN '+@field+' >= (@dttm-7) THEN ''WEEK''
                                   WHEN '+@field+' >= (@dttm-30) THEN ''MONTH''
                                   WHEN '+@field+' >= (@dttm-90) THEN ''QTR''
                                   WHEN '+@field+' >= (@dttm-365) THEN ''YEAR''
                                   ELSE ''NOSCOPE'' END AS addrange
                         FROM mdm_'+@parm_entity_type+'_tt) retquery
                WHERE ADDRANGE <> ''NOSCOPE''
               GROUP BY mdm_src_sys_id, addrange';                 
    EXECUTE sp_executesql @strSql, N'@dttm datetime',@parm_curr_dttm
        
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] @parm_entity_type,null,0,null,'mdm_dashboardGetActiveRetired',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_batchLoadStats]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_batchLoadStats]
GO

CREATE PROCEDURE [mdm_batchLoadStats] (@parm_entity_type     VARCHAR(30),
                                       @parm_process_id      INT) AS

SET NOCOUNT ON
DECLARE @strSql nvarchar(1000);
DECLARE @recordsAdded    INT;
DECLARE @clustersAdded   INT;
DECLARE @clustersUpdated INT;
DECLARE @clustersRetired INT;
DECLARE @stagingId       INT;
DECLARE @startTime       DATETIME;
DECLARE @endTime         DATETIME;
BEGIN TRY  
    SELECT @stagingId = mdm_staging_id,
           @startTime = process_start_dttm,
           @endTime = process_end_dttm
      FROM mdm_process_control WHERE process_id = @parm_process_id
    
    SET @strSql = 'SELECT @recordsAdded = COUNT(1) FROM mdm_'+@parm_entity_type+'_tt
                WHERE mdm_staging_id = @stgId and mdm_src_sys_id <> 1';                 
    EXECUTE sp_executesql @strSql, N'@stgId int, @recordsAdded INTEGER OUTPUT',@stagingId,@recordsAdded OUTPUT
    
    SET @strSql = 'SELECT @clustersAdded = COUNT(DISTINCT mdm_entity_cluster_id) FROM
                      (SELECT mdm_entity_cluster_id,
                              MIN(ISNULL(mdm_valid_from_dttm,CAST(''12/31/1000'' AS DATETIME)))
                              OVER (PARTITION BY mdm_entity_cluster_id) min_valid_from_dttm
                         FROM mdm_'+@parm_entity_type+'_tt WHERE mdm_survivor=''Y'') clusterQuery
                WHERE min_valid_from_dttm >= @startTime AND min_valid_from_dttm <= @endTime';                 
    EXECUTE sp_executesql @strSql, N'@startTime datetime,@endTime datetime,@clustersAdded INTEGER OUTPUT',@startTime,@endTime,@clustersAdded OUTPUT
    
    SET @strSql = 'SELECT @clustersUpdated = COUNT(1) FROM
                      (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm,
                              MAX(ISNULL(mdm_valid_to_dttm,CAST(''12/31/9999'' as datetime)))
                               OVER (PARTITION BY mdm_entity_cluster_id) max_valid_to_dttm
                         FROM mdm_'+@parm_entity_type+'_tt
                        WHERE mdm_survivor=''Y'') clusterQuery
                WHERE mdm_valid_to_dttm <> max_valid_to_dttm
                  AND mdm_valid_to_dttm >= @startTime and mdm_valid_to_dttm <= @endTime';                 
    EXECUTE sp_executesql @strSql, N'@startTime datetime,@endTime datetime,@clustersUpdated INTEGER OUTPUT',@startTime,@endTime,@clustersUpdated OUTPUT
    
    SET @strSql = 'SELECT @clustersRetired = COUNT(DISTINCT mdm_entity_cluster_id) FROM
                      (SELECT mdm_entity_cluster_id,
                              MAX(ISNULL(mdm_valid_to_dttm,CAST(''12/31/9999'' as datetime)))
                              OVER (PARTITION BY mdm_entity_cluster_id) max_valid_to_dttm
                         FROM mdm_'+@parm_entity_type+'_tt
                        WHERE mdm_survivor=''Y'') clusterQuery
                WHERE max_valid_to_dttm >= @startTime AND max_valid_to_dttm <= @endTime';                 
    EXECUTE sp_executesql @strSql, N'@startTime datetime,@endTime datetime,@clustersRetired INTEGER OUTPUT',@startTime,@endTime,@clustersRetired OUTPUT
    
    SELECT @recordsAdded AS records_added, @clustersAdded AS clusters_added, @clustersUpdated AS clusters_updated, @clustersRetired AS clusters_retired
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] @parm_entity_type,null,0,null,'mdm_batchLoadStats',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_dashboardCountsByStatus]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_dashboardCountsByStatus]
GO

CREATE PROCEDURE [mdm_dashboardCountsByStatus] (@parm_entity_type     VARCHAR(30)) AS

SET NOCOUNT ON
DECLARE @strSql nvarchar(1000);
BEGIN TRY      
    SET @strSql = 'SELECT COUNT(*) records, mdm_src_sys_id, status
                     FROM (SELECT mdm_src_sys_id,
                                  CASE WHEN mdm_valid_to_dttm IS NULL THEN ''ACTIVE''
                                     ELSE ''RETIRED'' END AS status
                             FROM mdm_'+@parm_entity_type+'_tt) rcquery
               GROUP BY mdm_src_sys_id, status';                 
    EXECUTE sp_executesql @strSql
        
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] @parm_entity_type,null,0,null,'mdm_dashboardCountsByStatus',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_dashboardClusterSize]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_dashboardClusterSize]
GO

CREATE PROCEDURE [mdm_dashboardClusterSize] (@parm_entity_type     VARCHAR(30)) AS

SET NOCOUNT ON
DECLARE @strSql nvarchar(1000);
BEGIN TRY      
    SET @strSql = 'SELECT MAX(records) AS maxct, MIN(records) AS minct, AVG(CAST(records AS REAL)) AS avsize, SUM(records) AS record_sum, COUNT(records) AS record_count
                     FROM (SELECT COUNT(*) AS records, mdm_entity_cluster_id
                             FROM mdm_'+@parm_entity_type+'_tt
                        WHERE mdm_valid_to_dttm IS NULL AND mdm_survivor <> ''Y''
                       GROUP BY  mdm_entity_cluster_id ) clusquery';                 
    EXECUTE sp_executesql @strSql
        
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] @parm_entity_type,null,0,null,'mdm_dashboardClusterSize',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_getClusterHistory]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_getClusterHistory]
GO

CREATE PROCEDURE [mdm_getClusterHistory] (@parm_entity_type     VARCHAR(30),
                                          @parm_cluster_id      INT,
                                          @parm_cluster_label   VARCHAR(30)) AS

SET NOCOUNT ON
DECLARE @strSql nvarchar(max);
BEGIN TRY      
    -- Start wrapper query
    SET @strSql = 'SELECT vw.mdm_entity_id,ss.mdm_src_sys_name as Source_System,vw.mdm_src_sys_rec_id,vw.label,vw.modified_by,vw.action,vw.modified_date,vw.user_comment
                 FROM mdm_src_sys ss,(';  
    
    -- Current cluster members insert history             
    SET @strSql = @strSql+'SELECT tt.mdm_entity_id,tt.mdm_src_sys_id,tt.mdm_src_sys_rec_id,tt.'+@parm_cluster_label+' as Label,pc.process_user AS Modified_By,''INSERT'' AS Action,tt.mdm_valid_from_dttm AS Modified_Date,null AS User_Comment
                   FROM mdm_process_control pc,
                        mdm_'+@parm_entity_type+'_tt tt,
                       (SELECT mdm_entity_id
                           FROM mdm_'+@parm_entity_type+'_tt
                          WHERE mdm_entity_cluster_id = @clusterId
                            AND mdm_entity_id NOT IN (SELECT DISTINCT tt.mdm_entity_id
                                                        FROM mdm_'+@parm_entity_type+'_tt tt, mdm_cluster_member_history cmh                        
                                                       WHERE tt.mdm_entity_cluster_id = @clusterId
                                                         AND tt.mdm_entity_id = cmh.mdm_entity_id)
                         UNION
                         SELECT cmh.mdm_entity_id
                           FROM mdm_cluster_member_history cmh
                          WHERE cmh.mdm_entity_cluster_id = @clusterId
                            AND cmh.mdm_entity_id NOT IN (SELECT cmh.mdm_entity_id
                                                            FROM mdm_cluster_member_history cmh,
                                                                 mdm_cluster_member_history prev  
                                                           WHERE cmh.mdm_entity_cluster_id = @clusterId
                                                             AND cmh.mdm_entity_id = prev.mdm_entity_id
                                                             AND prev.mdm_modified_dttm < cmh.mdm_modified_dttm)) ttid                        
                  WHERE ttid.mdm_entity_id = tt.mdm_entity_id
                    AND tt.mdm_staging_id = pc.mdm_staging_id
                    AND pc.process_name = ''MAKE DATA LIVE''  '; 
                
    -- Current cluster members retire history
    SET @strSql = @strSql+' UNION SELECT al.record_id AS mdm_entity_id,tt.mdm_src_sys_id,tt.mdm_src_sys_rec_id,tt.'+@parm_cluster_label+' AS Label,al.mdm_modified_by AS Modified_By,al.user_action AS Action,al.modified_dttm AS Modified_Date,null AS User_Comment
               FROM mdm_'+@parm_entity_type+'_al al,mdm_'+@parm_entity_type+'_tt tt
              WHERE tt.mdm_entity_cluster_id = @clusterId AND tt.mdm_entity_id = al.record_id '; 
              
    -- Previous cluster members that have been moved out or collapsed into another cluster
    SET @strSql = @strSql+' UNION SELECT cmh.mdm_entity_id,tt.mdm_src_sys_id,tt.mdm_src_sys_rec_id,tt.'+@parm_cluster_label+' as Label,null AS Modified_By,CASE WHEN (cmh.mdm_reason = ''M'' OR cmh.mdm_reason = ''R'') THEN ''MOVED (OUT)'' WHEN (cmh.mdm_reason = ''C'') THEN ''CLUSTER COLLAPSE (OUT)'' END AS Action,
                              cmh.mdm_modified_dttm AS Modified_Date,mdm_comment AS User_Comment
                         FROM mdm_cluster_member_history cmh,mdm_'+@parm_entity_type+'_tt tt
                        WHERE cmh.mdm_entity_cluster_id = @clusterId AND tt.mdm_entity_id = cmh.mdm_entity_id ';
                        
    -- Current cluster members that have moved in or collapsed from another cluster
    SET @strSql = @strSql+' UNION SELECT cmh.mdm_entity_id,tt.mdm_src_sys_id,tt.mdm_src_sys_rec_id,tt.'+@parm_cluster_label+' as Label,null AS Modified_By,CASE WHEN (cmh.mdm_reason = ''M'' OR cmh.mdm_reason = ''R'') THEN ''MOVED (IN)'' WHEN (cmh.mdm_reason = ''C'') THEN ''CLUSTER COLLAPSE (IN)'' END AS Action,
                           cmh.mdm_modified_dttm AS Modified_Date, cmh.mdm_comment AS User_Comment
                      FROM mdm_cluster_member_history cmh
                     INNER JOIN mdm_'+@parm_entity_type+'_tt tt ON tt.mdm_entity_id = cmh.mdm_entity_id                      
                     WHERE tt.mdm_entity_cluster_id = @clusterId
                       AND cmh.mdm_valid_to_dttm IS NULL ';
    
    -- Previous cluster members that have moved in or collapsed from another cluster and have sinced been moved out or collapsed into a different cluster
    SET @strSql = @strSql+' UNION SELECT cmh.mdm_entity_id,tt.mdm_src_sys_id,tt.mdm_src_sys_rec_id,tt.'+@parm_cluster_label+' AS Label,null AS Modified_By,CASE WHEN (cmh.mdm_reason = ''M'' OR cmh.mdm_reason = ''R'') THEN ''MOVED (IN)'' WHEN (cmh.mdm_reason = ''C'') THEN ''CLUSTER COLLAPSE (IN)'' END mdm_reason,
                              cmh.mdm_modified_dttm AS Modified_Date, cmh.mdm_comment AS User_Comment
                         FROM mdm_cluster_member_history cmh,mdm_'+@parm_entity_type+'_tt tt,
                              (SELECT cmh1.mdm_entity_id,max(cmh1.mdm_modified_dttm) over (partition by cmh1.mdm_entity_id) mdm_modified_dttm
                                 FROM mdm_cluster_member_history cmh1,mdm_cluster_member_history cmh2
                                WHERE cmh1.mdm_entity_cluster_id <> @clusterId AND cmh1.mdm_modified_dttm < cmh2.mdm_modified_dttm
                                  AND cmh1.mdm_entity_id = cmh2.mdm_entity_id AND cmh2.mdm_entity_cluster_id = @clusterId) temp
                        WHERE temp.mdm_entity_id = cmh.mdm_entity_id AND temp.mdm_modified_dttm = cmh.mdm_modified_dttm
                          AND cmh.mdm_entity_id = tt.mdm_entity_id ';                   
    
    -- Close wrapper query
    SET @strSql = @strSql+') vw WHERE vw.mdm_src_sys_id = ss.mdm_src_sys_id ORDER BY modified_date';               
                                
    EXECUTE sp_executesql @strSql,N'@clusterId  INT', @parm_cluster_id
END TRY 
BEGIN CATCH    
    DECLARE @error_messages VARCHAR(2000)
    DECLARE @errorSeverity INT
	DECLARE @errorState  INT
	DECLARE @errorStatus INT
	DECLARE @errorProc  VARCHAR(150)
	DECLARE @errorDate DATETIME

	DECLARE @mdm_error_status  VARCHAR(10)
	DECLARE @mdm_error_messages VARCHAR(4000)
	DECLARE @errorSeverityString VARCHAR(100)
	DECLARE @errorStateString VARCHAR(100)

	SELECT @errorStatus = ERROR_NUMBER()
	SELECT @error_messages = ERROR_MESSAGE()
	SELECT @errorSeverity = ERROR_SEVERITY()
	SELECT @errorState  = ERROR_STATE()
	SELECT @errorProc = ERROR_PROCEDURE()
	SELECT @errorDate = getDate()

	SELECT @errorSeverityString = CONVERT(VARCHAR,@errorSeverity)
	SELECT @errorStateString = CONVERT(VARCHAR,@errorState)
	SELECT @mdm_error_status = CONVERT(VARCHAR,@errorStatus)
	SELECT @mdm_error_messages = ERROR_PROCEDURE()+ ' : '+@error_messages+ 
									' : ' + CONVERT(VARCHAR,@errorSeverity)+
									' : ' + CONVERT(VARCHAR,ERROR_NUMBER())+
									' : '+CONVERT(VARCHAR,@errorState)
	-- Log Error
	EXEC [mdm_log_error] @parm_entity_type,'mdm_entity_cluster_id',@parm_cluster_id,null,'mdm_getClusterHistory',@mdm_error_status,@mdm_error_messages,@errorDate
	-- Raise Error
	RAISERROR (@error_messages, @errorSeverity, @errorState)
END CATCH 

GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[mdm_archive]') AND type in (N'P', N'PC'))
DROP PROCEDURE [mdm_archive]
GO

CREATE PROCEDURE [mdm_archive] (@parm_entity_type VARCHAR(23), @parm_curr_dttm DATETIME, @parm_action VARCHAR(20))
  AS
  
  DECLARE @sqlStr NVARCHAR(MAX);
  DECLARE @err_location INT;
  DECLARE @key uniqueidentifier;
  DECLARE @tt_columns NVARCHAR(MAX);
  DECLARE @entity_type_id INT;
  DECLARE @err_message VARCHAR(100);
  
  BEGIN
   SET NOCOUNT ON
  BEGIN TRY
    SET @err_location = 10
    SET @err_message = '@parm_entity_type:'+@parm_entity_type+':@parm_action:'+@parm_action
    EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','',@err_message,@parm_curr_dttm;
    IF (LOWER(@parm_action) = LOWER('COUNT'))   
     BEGIN 
		SET @sqlStr = 'SELECT COUNT(1)AS archival_count
						 FROM mdm_'+@parm_entity_type+'_tt
						WHERE mdm_valid_to_dttm <= @dttm';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Get archival count',@parm_curr_dttm;
     END
     
    SET @err_location = 20
    IF (LOWER(@parm_action) = LOWER('RELATIONSHIP_COUNT'))   
     BEGIN 
		SET @sqlStr = 'SELECT COUNT(*) AS relationship_count FROM
						(SELECT DISTINCT rel.mdm_from_cluster_id, rel.mdm_to_cluster_id
						   FROM mdm_entity_rel rel, mdm_'+@parm_entity_type+'_tt tt
						  WHERE rel.mdm_valid_to_dttm <= @dttm
							AND rel.mdm_valid_from_dttm <> rel.mdm_valid_to_dttm
							AND (tt.mdm_entity_cluster_id = rel.mdm_from_cluster_id OR tt.mdm_entity_cluster_id = rel.mdm_to_cluster_id)) t';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Get relationship count',@parm_curr_dttm;
     END  
     
    SET @err_location = 30
    IF (LOWER(@parm_action) = LOWER('GET_AUDIT'))   
     BEGIN 
		SET @sqlStr = 'SELECT * FROM mdm_'+@parm_entity_type+'_al
						WHERE modified_dttm <= @dttm';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Get audit records',@parm_curr_dttm;
     END        
     
    SET @err_location = 40
    IF (LOWER(@parm_action) = LOWER('GET_HISTORY'))   
     BEGIN 
		SET @sqlStr = 'SELECT hist.* FROM mdm_cluster_member_history hist, mdm_'+@parm_entity_type+'_tt tt
						WHERE hist.mdm_entity_id = tt.mdm_entity_id
						  AND tt.mdm_valid_to_dttm <= @dttm
					    UNION 
					   SELECT * FROM mdm_cluster_member_history
						WHERE EXISTS (SELECT mdm_entity_cluster_id 
										FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm,
												     MAX(ISNULL(mdm_valid_to_dttm,CAST(''12/31/9999'' AS DATETIME)))
												     OVER (PARTITION BY mdm_entity_cluster_id) max_valid_to_dttm
											    FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm FROM mdm_'+@parm_entity_type+'_tt 
													   WHERE mdm_survivor=''Y''
						                         AND (mdm_valid_to_dttm IS NULL OR mdm_valid_to_dttm <= @dttm)) tmp) tmp1
									   WHERE mdm_valid_to_dttm IS NOT NULL AND mdm_valid_to_dttm = max_valid_to_dttm
								         AND mdm_cluster_member_history.mdm_entity_cluster_id = mdm_entity_cluster_id)';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Get history records',@parm_curr_dttm;
     END   
     
    SET @err_location = 50
    IF (LOWER(@parm_action) = LOWER('GET_RELATIONSHIPS'))   
     BEGIN 
		SET @sqlStr = 'SELECT rel.* FROM mdm_entity_rel rel,
						  (SELECT DISTINCT rel.mdm_from_cluster_id, rel.mdm_to_cluster_id
							 FROM mdm_entity_rel rel, mdm_'+@parm_entity_type+'_tt tt
							WHERE rel.mdm_valid_to_dttm <= @dttm
							  AND rel.mdm_valid_from_dttm <> rel.mdm_valid_to_dttm
							  AND (tt.mdm_entity_cluster_id = rel.mdm_from_cluster_id OR tt.mdm_entity_cluster_id = rel.mdm_to_cluster_id)) tmp
					 WHERE rel.mdm_from_cluster_id = tmp.mdm_from_cluster_id
					   AND rel.mdm_to_cluster_id = tmp.mdm_to_cluster_id';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Get relationships',@parm_curr_dttm;
     END  
     
    SET @err_location = 60
    IF (LOWER(@parm_action) = LOWER('GET_MAPPINGS'))   
     BEGIN 
		SET @sqlStr = 'SELECT cm.* FROM mdm_cluster_mapping cm,
						   (SELECT mdm_entity_cluster_id 
							  FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm,
										   MAX(ISNULL(mdm_valid_to_dttm,CAST(''12/31/9999'' AS DATETIME)))
										   OVER (PARTITION BY mdm_entity_cluster_id) max_valid_to_dttm
									  FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm FROM mdm_'+@parm_entity_type+'_tt 
									         WHERE mdm_survivor=''Y''
									           AND (mdm_valid_to_dttm IS NULL OR mdm_valid_to_dttm <= @dttm)) tmp) tmp1
							 WHERE mdm_valid_to_dttm IS NOT NULL
							   AND mdm_valid_to_dttm = max_valid_to_dttm) ret_clusters
					 WHERE cm.original_cluster_id = ret_clusters.mdm_entity_cluster_id
						OR cm.current_cluster_id = ret_clusters.mdm_entity_cluster_id';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Get cluster mappings',@parm_curr_dttm;
     END 
     
    SET @err_location = 70
    IF (LOWER(@parm_action) = LOWER('GET_CLUSTERS'))   
     BEGIN 
		SET @sqlStr = 'SELECT ec.* FROM mdm_entity_clusters ec,
                           (SELECT mdm_entity_cluster_id 
							  FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm,
										   MAX(ISNULL(mdm_valid_to_dttm,CAST(''12/31/9999'' AS DATETIME)))
										   OVER (PARTITION BY mdm_entity_cluster_id) max_valid_to_dttm
									  FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm FROM mdm_'+@parm_entity_type+'_tt 
									         WHERE mdm_survivor=''Y''
									           AND (mdm_valid_to_dttm IS NULL OR mdm_valid_to_dttm <= @dttm)) tmp) tmp1
							 WHERE mdm_valid_to_dttm IS NOT NULL
							   AND mdm_valid_to_dttm = max_valid_to_dttm) ret_clusters
                       WHERE ec.mdm_entity_cluster_id = ret_clusters.mdm_entity_cluster_id';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Get entity clusters',@parm_curr_dttm;
     END  
     
    SET @err_location = 80
    IF (LOWER(@parm_action) = LOWER('GET_ENTITIES'))   
     BEGIN 
        SELECT @entity_type_id = mdm_entity_type_id FROM mdm_entity_types
         WHERE LOWER(mdm_entity_type) = LOWER(@parm_entity_type)
         
        CREATE TABLE #colStr (colStrValue nvarchar(max));
    
		INSERT INTO #colStr   
	    
		EXEC [mdm_getDecryptTtCols] @entity_type_id    
		SELECT @tt_columns = colStrValue FROM #colStr
        
        DROP TABLE #colStr
    
		OPEN SYMMETRIC KEY qMDMKey DECRYPTION BY CERTIFICATE qMDMCer
		
		SET @sqlStr = 'SELECT mdm_staging_id,mdm_entity_id,mdm_src_sys_id,mdm_src_sys_rec_id,mdm_language_id,mdm_entity_cluster_id,'+
		                     'mdm_survivor,mdm_valid_from_dttm,mdm_valid_to_dttm'++@tt_columns+' '+
		               ' FROM mdm_'+@parm_entity_type+'_tt WHERE mdm_valid_to_dttm <= @dttm';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Get transpose records',@parm_curr_dttm;
				
		CLOSE SYMMETRIC KEY qMDMKey
     END     
    
    IF (LOWER(@parm_action) = LOWER('DELETE'))   
     BEGIN 
      BEGIN TRY
       EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Starting Delete',@parm_curr_dttm;
       BEGIN TRAN 
		SET @err_location = 90
		SET @sqlStr = 'DELETE  FROM mdm_'+@parm_entity_type+'_al
						WHERE modified_dttm <= @dttm';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Deleted from audit log',@parm_curr_dttm;
		
		SET @err_location = 100
		SET @sqlStr = 'DELETE FROM mdm_cluster_member_history 
						WHERE EXISTS (SELECT 1 FROM mdm_'+@parm_entity_type+'_tt tt
									   WHERE tt.mdm_valid_to_dttm <= @dttm
										 AND mdm_cluster_member_history.mdm_entity_id = tt.mdm_entity_id)';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		
		SET @sqlStr = 'DELETE FROM mdm_cluster_member_history
						WHERE EXISTS (SELECT mdm_entity_cluster_id 
										FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm,
												     MAX(ISNULL(mdm_valid_to_dttm,CAST(''12/31/9999'' AS DATETIME)))
												     OVER (PARTITION BY mdm_entity_cluster_id) max_valid_to_dttm
											    FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm FROM mdm_'+@parm_entity_type+'_tt 
													   WHERE mdm_survivor=''Y''
						                         AND (mdm_valid_to_dttm IS NULL OR mdm_valid_to_dttm <= @dttm)) tmp) tmp1
									   WHERE mdm_valid_to_dttm IS NOT NULL AND mdm_valid_to_dttm = max_valid_to_dttm
								         AND mdm_cluster_member_history.mdm_entity_cluster_id = mdm_entity_cluster_id)';
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Deleted from cluster member history',@parm_curr_dttm;
		
		SET @err_location = 110
		SET @sqlStr = 'DELETE FROM mdm_entity_rel 
						WHERE EXISTS (SELECT 1 
										FROM (SELECT DISTINCT rel1.mdm_from_cluster_id, rel1.mdm_to_cluster_id
												 FROM mdm_entity_rel rel1, mdm_'+@parm_entity_type+'_tt tt
												WHERE rel1.mdm_valid_to_dttm <= @dttm
												  AND rel1.mdm_valid_from_dttm <> rel1.mdm_valid_to_dttm
												  AND (tt.mdm_entity_cluster_id = rel1.mdm_from_cluster_id OR tt.mdm_entity_cluster_id = rel1.mdm_to_cluster_id)) tmp
											WHERE mdm_entity_rel.mdm_from_cluster_id = tmp.mdm_from_cluster_id
											  AND mdm_entity_rel.mdm_to_cluster_id = tmp.mdm_to_cluster_id)';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Deleted from entity rel',@parm_curr_dttm;
		
		SET @err_location = 120
		SET @sqlStr = 'DELETE FROM mdm_cluster_mapping
						WHERE EXISTS (SELECT mdm_entity_cluster_id 
										FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm,
												     MAX(ISNULL(mdm_valid_to_dttm,CAST(''12/31/9999'' AS DATETIME)))
												     OVER (PARTITION BY mdm_entity_cluster_id) max_valid_to_dttm
											    FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm FROM mdm_'+@parm_entity_type+'_tt 
													   WHERE mdm_survivor=''Y''
						                         AND (mdm_valid_to_dttm IS NULL OR mdm_valid_to_dttm <= @dttm)) tmp) tmp1
									   WHERE mdm_valid_to_dttm IS NOT NULL AND mdm_valid_to_dttm = max_valid_to_dttm
								         AND (mdm_cluster_mapping.original_cluster_id = mdm_entity_cluster_id
											  OR mdm_cluster_mapping.current_cluster_id = mdm_entity_cluster_id))';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Deleted from cluster mapping',@parm_curr_dttm;
		
		SET @err_location = 130
		SET @sqlStr = 'DELETE FROM mdm_entity_clusters
						WHERE EXISTS (SELECT mdm_entity_cluster_id 
										FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm,
												     MAX(ISNULL(mdm_valid_to_dttm,CAST(''12/31/9999'' AS DATETIME)))
												     OVER (PARTITION BY mdm_entity_cluster_id) max_valid_to_dttm
											    FROM (SELECT mdm_entity_cluster_id,mdm_valid_to_dttm FROM mdm_'+@parm_entity_type+'_tt 
													   WHERE mdm_survivor=''Y''
						                         AND (mdm_valid_to_dttm IS NULL OR mdm_valid_to_dttm <= @dttm)) tmp) tmp1
									   WHERE mdm_valid_to_dttm IS NOT NULL AND mdm_valid_to_dttm = max_valid_to_dttm
								         AND mdm_entity_clusters.mdm_entity_cluster_id = mdm_entity_cluster_id)';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Deleted from entity clusters',@parm_curr_dttm;
		
		
		SET @err_location = 140
		SET @sqlStr = 'DELETE FROM mdm_'+@parm_entity_type+'_tt
					    WHERE mdm_valid_to_dttm <= @dttm';						
		EXECUTE sp_executesql @sqlStr,N'@dttm datetime',@parm_curr_dttm
		EXECUTE mdm_log_error 'MDM_ARCHIVE',null,0,null,'mdm_archive','','Deleted from transpose',@parm_curr_dttm;
	   COMMIT TRAN 
	  END TRY
	  BEGIN CATCH
	   ROLLBACK TRAN
	   DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
		SET @error_messages='Location : '+CONVERT(VARCHAR,@err_location)+':'+ERROR_MESSAGE();
		SET @errorSeverity=ERROR_SEVERITY();
		SET @errorState=ERROR_STATE();
		SET @errorStatus=ERROR_NUMBER();
		SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
		set @dttm=getdate()
		SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
		EXEC [mdm_log_error] @parm_entity_type,null,0,null,'mdm_archive',@strErrorStatus,@mdm_error_messages,@dttm
		RAISERROR(@error_messages, @errorSeverity, @errorState)
	  END CATCH
     END
  END TRY
  BEGIN CATCH    
    --DECLARE @mdm_error_messages VARCHAR(4000),@error_messages VARCHAR(2000),@errorStatus INT,@strErrorStatus varchar(10),@errorSeverity INT,@errorState  INT,@dttm DATETIME;
    SET @error_messages='Location : '+CONVERT(VARCHAR,@err_location)+':'+ERROR_MESSAGE();
    SET @errorSeverity=ERROR_SEVERITY();
    SET @errorState=ERROR_STATE();
    SET @errorStatus=ERROR_NUMBER();
    SET @strErrorStatus=CONVERT(varchar(10), @errorStatus);
    set @dttm=getdate()
    SET @mdm_error_messages = ERROR_PROCEDURE()+ ': '+@error_messages+' ('+CONVERT(VARCHAR,@errorSeverity)+' : '+CONVERT(VARCHAR,@errorStatus)+' : '+CONVERT(VARCHAR,@errorState)+')'
    EXEC [mdm_log_error] @parm_entity_type,null,0,null,'mdm_archive',@strErrorStatus,@mdm_error_messages,@dttm
    RAISERROR(@error_messages, @errorSeverity, @errorState)
  END CATCH
  END


GO


IF  EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[MDM_ENTITY_TYPES_BU]'))
DROP TRIGGER [MDM_ENTITY_TYPES_BU]
GO

CREATE TRIGGER [MDM_ENTITY_TYPES_BU] 
    ON  [MDM_ENTITY_TYPES] 
AFTER UPDATE
AS 
BEGIN
	SET NOCOUNT ON;
	UPDATE MDM_ENTITY_TYPES set mdm_last_mod_dttm=getdate() where mdm_entity_type_id IN (select mdm_entity_type_id from inserted); 
END

GO

IF EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[MDM_PROCESS_CONTROL_IOI]') )
	DROP TRIGGER MDM_PROCESS_CONTROL_IOI
GO

CREATE TRIGGER [MDM_PROCESS_CONTROL_IOI] 
    ON  [MDM_PROCESS_CONTROL] 
INSTEAD OF INSERT
AS 
BEGIN
  SET NOCOUNT ON;
  declare @SEQ_CURR_VAL INT,@PROCESS_ID int,@MDM_STAGING_ID int,@PROCESS_NAME varchar(50),@MDM_ENTITY_TYPE_ID int,@PROCESS_RUNNING varchar(1),@PROCESS_SUCCESS varchar(1),
    @PROCESS_USER varchar(50),@PROCESS_START_DTTM DATETIME,@PROCESS_END_DTTM DATETIME
  declare insCursor CURSOR LOCAL FOR SELECT PROCESS_ID,MDM_STAGING_ID,PROCESS_NAME,MDM_ENTITY_TYPE_ID,PROCESS_RUNNING,PROCESS_SUCCESS,PROCESS_USER,PROCESS_START_DTTM,PROCESS_END_DTTM FROM inserted
  OPEN insCursor
  FETCH insCursor into @PROCESS_ID,@MDM_STAGING_ID,@PROCESS_NAME,@MDM_ENTITY_TYPE_ID,@PROCESS_RUNNING,@PROCESS_SUCCESS,@PROCESS_USER,@PROCESS_START_DTTM,@PROCESS_END_DTTM
  while (@@FETCH_STATUS=0)
  BEGIN
    if @process_id is null or @process_id = 0
      exec @process_id=mdm_get_sequence_next N'MDM_PROCESS_CONTROL'
    INSERT INTO MDM_PROCESS_CONTROL (PROCESS_ID,MDM_STAGING_ID,PROCESS_NAME,MDM_ENTITY_TYPE_ID,PROCESS_RUNNING,PROCESS_SUCCESS,PROCESS_USER,PROCESS_START_DTTM,PROCESS_END_DTTM)
    values (@PROCESS_ID,@MDM_STAGING_ID,@PROCESS_NAME,@MDM_ENTITY_TYPE_ID,@PROCESS_RUNNING,@PROCESS_SUCCESS,@PROCESS_USER,@PROCESS_START_DTTM,@PROCESS_END_DTTM)
    FETCH insCursor into @PROCESS_ID,@MDM_STAGING_ID,@PROCESS_NAME,@MDM_ENTITY_TYPE_ID,@PROCESS_RUNNING,@PROCESS_SUCCESS,@PROCESS_USER,@PROCESS_START_DTTM,@PROCESS_END_DTTM
  END
  CLOSE insCursor
  DEALLOCATE insCursor
END
GO
